from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class AreaUnits(Enum):
        """
            AreaUnits enumeration
        """
        
        SquareKilometer = 'SquareKilometer'
        """
            
        """
        
        SquareMeter = 'SquareMeter'
        """
            
        """
        
        SquareDecimeter = 'SquareDecimeter'
        """
            
        """
        
        SquareCentimeter = 'SquareCentimeter'
        """
            
        """
        
        SquareMillimeter = 'SquareMillimeter'
        """
            
        """
        
        SquareMicrometer = 'SquareMicrometer'
        """
            
        """
        
        SquareMile = 'SquareMile'
        """
            The statute mile was standardised between the British Commonwealth and the United States by an international agreement in 1959, when it was formally redefined with respect to SI units as exactly 1,609.344 metres.
        """
        
        SquareYard = 'SquareYard'
        """
            The yard (symbol: yd) is an English unit of length in both the British imperial and US customary systems of measurement equalling 3 feet (or 36 inches). Since 1959 the yard has been by international agreement standardized as exactly 0.9144 meter. A distance of 1,760 yards is equal to 1 mile.
        """
        
        SquareFoot = 'SquareFoot'
        """
            
        """
        
        UsSurveySquareFoot = 'UsSurveySquareFoot'
        """
            In the United States, the foot was defined as 12 inches, with the inch being defined by the Mendenhall Order of 1893 as 39.37 inches = 1 m. This makes a U.S. survey foot exactly 1200/3937 meters.
        """
        
        SquareInch = 'SquareInch'
        """
            
        """
        
        Acre = 'Acre'
        """
            Based upon the international yard and pound agreement of 1959, an acre may be declared as exactly 4,046.8564224 square metres.
        """
        
        Hectare = 'Hectare'
        """
            
        """
        
        SquareNauticalMile = 'SquareNauticalMile'
        """
            
        """
        

class AreaDto:
    """
    A DTO representation of a Area

    Attributes:
        value (float): The value of the Area.
        unit (AreaUnits): The specific unit that the Area value is representing.
    """

    def __init__(self, value: float, unit: AreaUnits):
        """
        Create a new DTO representation of a Area

        Parameters:
            value (float): The value of the Area.
            unit (AreaUnits): The specific unit that the Area value is representing.
        """
        self.value: float = value
        """
        The value of the Area
        """
        self.unit: AreaUnits = unit
        """
        The specific unit that the Area value is representing
        """

    def to_json(self):
        """
        Get a Area DTO JSON object representing the current unit.

        :return: JSON object represents Area DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Area DTO from a json representation.

        :param data: The Area DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeter"}
        :return: A new instance of AreaDto.
        :rtype: AreaDto
        """
        return AreaDto(value=data["value"], unit=AreaUnits(data["unit"]))


class Area(AbstractMeasure):
    """
    Area is a quantity that expresses the extent of a two-dimensional surface or shape, or planar lamina, in the plane. Area can be understood as the amount of material with a given thickness that would be necessary to fashion a model of the shape, or the amount of paint necessary to cover the surface with a single coat.[1] It is the two-dimensional analog of the length of a curve (a one-dimensional concept) or the volume of a solid (a three-dimensional concept).

    Args:
        value (float): The value.
        from_unit (AreaUnits): The Area unit to create from, The default unit is SquareMeter
    """
    def __init__(self, value: float, from_unit: AreaUnits = AreaUnits.SquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__square_kilometers = None
        
        self.__square_meters = None
        
        self.__square_decimeters = None
        
        self.__square_centimeters = None
        
        self.__square_millimeters = None
        
        self.__square_micrometers = None
        
        self.__square_miles = None
        
        self.__square_yards = None
        
        self.__square_feet = None
        
        self.__us_survey_square_feet = None
        
        self.__square_inches = None
        
        self.__acres = None
        
        self.__hectares = None
        
        self.__square_nautical_miles = None
        

    def convert(self, unit: AreaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: AreaUnits = AreaUnits.SquareMeter) -> AreaDto:
        """
        Get a new instance of Area DTO representing the current unit.

        :param hold_in_unit: The specific Area unit to store the Area value in the DTO representation.
        :type hold_in_unit: AreaUnits
        :return: A new instance of AreaDto.
        :rtype: AreaDto
        """
        return AreaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: AreaUnits = AreaUnits.SquareMeter):
        """
        Get a Area DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Area unit to store the Area value in the DTO representation.
        :type hold_in_unit: AreaUnits
        :return: JSON object represents Area DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(area_dto: AreaDto):
        """
        Obtain a new instance of Area from a DTO unit object.

        :param area_dto: The Area DTO representation.
        :type area_dto: AreaDto
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(area_dto.value, area_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Area from a DTO unit json representation.

        :param data: The Area DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SquareMeter"}
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area.from_dto(AreaDto.from_json(data))

    def __convert_from_base(self, from_unit: AreaUnits) -> float:
        value = self._value
        
        if from_unit == AreaUnits.SquareKilometer:
            return (value / 1e6)
        
        if from_unit == AreaUnits.SquareMeter:
            return (value)
        
        if from_unit == AreaUnits.SquareDecimeter:
            return (value / 1e-2)
        
        if from_unit == AreaUnits.SquareCentimeter:
            return (value / 1e-4)
        
        if from_unit == AreaUnits.SquareMillimeter:
            return (value / 1e-6)
        
        if from_unit == AreaUnits.SquareMicrometer:
            return (value / 1e-12)
        
        if from_unit == AreaUnits.SquareMile:
            return (value / 1609.344 / 1609.344)
        
        if from_unit == AreaUnits.SquareYard:
            return (value / 0.9144 / 0.9144)
        
        if from_unit == AreaUnits.SquareFoot:
            return (value / 9.290304e-2)
        
        if from_unit == AreaUnits.UsSurveySquareFoot:
            return (value / (1200.0 / 3937.0) / (1200.0 / 3937.0))
        
        if from_unit == AreaUnits.SquareInch:
            return (value / 0.00064516)
        
        if from_unit == AreaUnits.Acre:
            return (value / 4046.8564224)
        
        if from_unit == AreaUnits.Hectare:
            return (value / 1e4)
        
        if from_unit == AreaUnits.SquareNauticalMile:
            return (value / 3429904)
        
        return None


    def __convert_to_base(self, value: float, to_unit: AreaUnits) -> float:
        
        if to_unit == AreaUnits.SquareKilometer:
            return (value * 1e6)
        
        if to_unit == AreaUnits.SquareMeter:
            return (value)
        
        if to_unit == AreaUnits.SquareDecimeter:
            return (value * 1e-2)
        
        if to_unit == AreaUnits.SquareCentimeter:
            return (value * 1e-4)
        
        if to_unit == AreaUnits.SquareMillimeter:
            return (value * 1e-6)
        
        if to_unit == AreaUnits.SquareMicrometer:
            return (value * 1e-12)
        
        if to_unit == AreaUnits.SquareMile:
            return (value * 1609.344 * 1609.344)
        
        if to_unit == AreaUnits.SquareYard:
            return (value * 0.9144 * 0.9144)
        
        if to_unit == AreaUnits.SquareFoot:
            return (value * 9.290304e-2)
        
        if to_unit == AreaUnits.UsSurveySquareFoot:
            return (value * (1200.0 / 3937.0) * (1200.0 / 3937.0))
        
        if to_unit == AreaUnits.SquareInch:
            return (value * 0.00064516)
        
        if to_unit == AreaUnits.Acre:
            return (value * 4046.8564224)
        
        if to_unit == AreaUnits.Hectare:
            return (value * 1e4)
        
        if to_unit == AreaUnits.SquareNauticalMile:
            return (value * 3429904)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_square_kilometers(square_kilometers: float):
        """
        Create a new instance of Area from a value in square_kilometers.

        

        :param meters: The Area value in square_kilometers.
        :type square_kilometers: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_kilometers, AreaUnits.SquareKilometer)

    
    @staticmethod
    def from_square_meters(square_meters: float):
        """
        Create a new instance of Area from a value in square_meters.

        

        :param meters: The Area value in square_meters.
        :type square_meters: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_meters, AreaUnits.SquareMeter)

    
    @staticmethod
    def from_square_decimeters(square_decimeters: float):
        """
        Create a new instance of Area from a value in square_decimeters.

        

        :param meters: The Area value in square_decimeters.
        :type square_decimeters: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_decimeters, AreaUnits.SquareDecimeter)

    
    @staticmethod
    def from_square_centimeters(square_centimeters: float):
        """
        Create a new instance of Area from a value in square_centimeters.

        

        :param meters: The Area value in square_centimeters.
        :type square_centimeters: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_centimeters, AreaUnits.SquareCentimeter)

    
    @staticmethod
    def from_square_millimeters(square_millimeters: float):
        """
        Create a new instance of Area from a value in square_millimeters.

        

        :param meters: The Area value in square_millimeters.
        :type square_millimeters: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_millimeters, AreaUnits.SquareMillimeter)

    
    @staticmethod
    def from_square_micrometers(square_micrometers: float):
        """
        Create a new instance of Area from a value in square_micrometers.

        

        :param meters: The Area value in square_micrometers.
        :type square_micrometers: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_micrometers, AreaUnits.SquareMicrometer)

    
    @staticmethod
    def from_square_miles(square_miles: float):
        """
        Create a new instance of Area from a value in square_miles.

        The statute mile was standardised between the British Commonwealth and the United States by an international agreement in 1959, when it was formally redefined with respect to SI units as exactly 1,609.344 metres.

        :param meters: The Area value in square_miles.
        :type square_miles: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_miles, AreaUnits.SquareMile)

    
    @staticmethod
    def from_square_yards(square_yards: float):
        """
        Create a new instance of Area from a value in square_yards.

        The yard (symbol: yd) is an English unit of length in both the British imperial and US customary systems of measurement equalling 3 feet (or 36 inches). Since 1959 the yard has been by international agreement standardized as exactly 0.9144 meter. A distance of 1,760 yards is equal to 1 mile.

        :param meters: The Area value in square_yards.
        :type square_yards: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_yards, AreaUnits.SquareYard)

    
    @staticmethod
    def from_square_feet(square_feet: float):
        """
        Create a new instance of Area from a value in square_feet.

        

        :param meters: The Area value in square_feet.
        :type square_feet: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_feet, AreaUnits.SquareFoot)

    
    @staticmethod
    def from_us_survey_square_feet(us_survey_square_feet: float):
        """
        Create a new instance of Area from a value in us_survey_square_feet.

        In the United States, the foot was defined as 12 inches, with the inch being defined by the Mendenhall Order of 1893 as 39.37 inches = 1 m. This makes a U.S. survey foot exactly 1200/3937 meters.

        :param meters: The Area value in us_survey_square_feet.
        :type us_survey_square_feet: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(us_survey_square_feet, AreaUnits.UsSurveySquareFoot)

    
    @staticmethod
    def from_square_inches(square_inches: float):
        """
        Create a new instance of Area from a value in square_inches.

        

        :param meters: The Area value in square_inches.
        :type square_inches: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_inches, AreaUnits.SquareInch)

    
    @staticmethod
    def from_acres(acres: float):
        """
        Create a new instance of Area from a value in acres.

        Based upon the international yard and pound agreement of 1959, an acre may be declared as exactly 4,046.8564224 square metres.

        :param meters: The Area value in acres.
        :type acres: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(acres, AreaUnits.Acre)

    
    @staticmethod
    def from_hectares(hectares: float):
        """
        Create a new instance of Area from a value in hectares.

        

        :param meters: The Area value in hectares.
        :type hectares: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(hectares, AreaUnits.Hectare)

    
    @staticmethod
    def from_square_nautical_miles(square_nautical_miles: float):
        """
        Create a new instance of Area from a value in square_nautical_miles.

        

        :param meters: The Area value in square_nautical_miles.
        :type square_nautical_miles: float
        :return: A new instance of Area.
        :rtype: Area
        """
        return Area(square_nautical_miles, AreaUnits.SquareNauticalMile)

    
    @property
    def square_kilometers(self) -> float:
        """
        
        """
        if self.__square_kilometers != None:
            return self.__square_kilometers
        self.__square_kilometers = self.__convert_from_base(AreaUnits.SquareKilometer)
        return self.__square_kilometers

    
    @property
    def square_meters(self) -> float:
        """
        
        """
        if self.__square_meters != None:
            return self.__square_meters
        self.__square_meters = self.__convert_from_base(AreaUnits.SquareMeter)
        return self.__square_meters

    
    @property
    def square_decimeters(self) -> float:
        """
        
        """
        if self.__square_decimeters != None:
            return self.__square_decimeters
        self.__square_decimeters = self.__convert_from_base(AreaUnits.SquareDecimeter)
        return self.__square_decimeters

    
    @property
    def square_centimeters(self) -> float:
        """
        
        """
        if self.__square_centimeters != None:
            return self.__square_centimeters
        self.__square_centimeters = self.__convert_from_base(AreaUnits.SquareCentimeter)
        return self.__square_centimeters

    
    @property
    def square_millimeters(self) -> float:
        """
        
        """
        if self.__square_millimeters != None:
            return self.__square_millimeters
        self.__square_millimeters = self.__convert_from_base(AreaUnits.SquareMillimeter)
        return self.__square_millimeters

    
    @property
    def square_micrometers(self) -> float:
        """
        
        """
        if self.__square_micrometers != None:
            return self.__square_micrometers
        self.__square_micrometers = self.__convert_from_base(AreaUnits.SquareMicrometer)
        return self.__square_micrometers

    
    @property
    def square_miles(self) -> float:
        """
        The statute mile was standardised between the British Commonwealth and the United States by an international agreement in 1959, when it was formally redefined with respect to SI units as exactly 1,609.344 metres.
        """
        if self.__square_miles != None:
            return self.__square_miles
        self.__square_miles = self.__convert_from_base(AreaUnits.SquareMile)
        return self.__square_miles

    
    @property
    def square_yards(self) -> float:
        """
        The yard (symbol: yd) is an English unit of length in both the British imperial and US customary systems of measurement equalling 3 feet (or 36 inches). Since 1959 the yard has been by international agreement standardized as exactly 0.9144 meter. A distance of 1,760 yards is equal to 1 mile.
        """
        if self.__square_yards != None:
            return self.__square_yards
        self.__square_yards = self.__convert_from_base(AreaUnits.SquareYard)
        return self.__square_yards

    
    @property
    def square_feet(self) -> float:
        """
        
        """
        if self.__square_feet != None:
            return self.__square_feet
        self.__square_feet = self.__convert_from_base(AreaUnits.SquareFoot)
        return self.__square_feet

    
    @property
    def us_survey_square_feet(self) -> float:
        """
        In the United States, the foot was defined as 12 inches, with the inch being defined by the Mendenhall Order of 1893 as 39.37 inches = 1 m. This makes a U.S. survey foot exactly 1200/3937 meters.
        """
        if self.__us_survey_square_feet != None:
            return self.__us_survey_square_feet
        self.__us_survey_square_feet = self.__convert_from_base(AreaUnits.UsSurveySquareFoot)
        return self.__us_survey_square_feet

    
    @property
    def square_inches(self) -> float:
        """
        
        """
        if self.__square_inches != None:
            return self.__square_inches
        self.__square_inches = self.__convert_from_base(AreaUnits.SquareInch)
        return self.__square_inches

    
    @property
    def acres(self) -> float:
        """
        Based upon the international yard and pound agreement of 1959, an acre may be declared as exactly 4,046.8564224 square metres.
        """
        if self.__acres != None:
            return self.__acres
        self.__acres = self.__convert_from_base(AreaUnits.Acre)
        return self.__acres

    
    @property
    def hectares(self) -> float:
        """
        
        """
        if self.__hectares != None:
            return self.__hectares
        self.__hectares = self.__convert_from_base(AreaUnits.Hectare)
        return self.__hectares

    
    @property
    def square_nautical_miles(self) -> float:
        """
        
        """
        if self.__square_nautical_miles != None:
            return self.__square_nautical_miles
        self.__square_nautical_miles = self.__convert_from_base(AreaUnits.SquareNauticalMile)
        return self.__square_nautical_miles

    
    def to_string(self, unit: AreaUnits = AreaUnits.SquareMeter, fractional_digits: int = None) -> str:
        """
        Format the Area to a string.
        
        Note: the default format for Area is SquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Area. Default is 'SquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == AreaUnits.SquareKilometer:
            return f"""{super()._truncate_fraction_digits(self.square_kilometers, fractional_digits)} km²"""
        
        if unit == AreaUnits.SquareMeter:
            return f"""{super()._truncate_fraction_digits(self.square_meters, fractional_digits)} m²"""
        
        if unit == AreaUnits.SquareDecimeter:
            return f"""{super()._truncate_fraction_digits(self.square_decimeters, fractional_digits)} dm²"""
        
        if unit == AreaUnits.SquareCentimeter:
            return f"""{super()._truncate_fraction_digits(self.square_centimeters, fractional_digits)} cm²"""
        
        if unit == AreaUnits.SquareMillimeter:
            return f"""{super()._truncate_fraction_digits(self.square_millimeters, fractional_digits)} mm²"""
        
        if unit == AreaUnits.SquareMicrometer:
            return f"""{super()._truncate_fraction_digits(self.square_micrometers, fractional_digits)} µm²"""
        
        if unit == AreaUnits.SquareMile:
            return f"""{super()._truncate_fraction_digits(self.square_miles, fractional_digits)} mi²"""
        
        if unit == AreaUnits.SquareYard:
            return f"""{super()._truncate_fraction_digits(self.square_yards, fractional_digits)} yd²"""
        
        if unit == AreaUnits.SquareFoot:
            return f"""{super()._truncate_fraction_digits(self.square_feet, fractional_digits)} ft²"""
        
        if unit == AreaUnits.UsSurveySquareFoot:
            return f"""{super()._truncate_fraction_digits(self.us_survey_square_feet, fractional_digits)} ft² (US)"""
        
        if unit == AreaUnits.SquareInch:
            return f"""{super()._truncate_fraction_digits(self.square_inches, fractional_digits)} in²"""
        
        if unit == AreaUnits.Acre:
            return f"""{super()._truncate_fraction_digits(self.acres, fractional_digits)} ac"""
        
        if unit == AreaUnits.Hectare:
            return f"""{super()._truncate_fraction_digits(self.hectares, fractional_digits)} ha"""
        
        if unit == AreaUnits.SquareNauticalMile:
            return f"""{super()._truncate_fraction_digits(self.square_nautical_miles, fractional_digits)} nmi²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: AreaUnits = AreaUnits.SquareMeter) -> str:
        """
        Get Area unit abbreviation.
        Note! the default abbreviation for Area is SquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == AreaUnits.SquareKilometer:
            return """km²"""
        
        if unit_abbreviation == AreaUnits.SquareMeter:
            return """m²"""
        
        if unit_abbreviation == AreaUnits.SquareDecimeter:
            return """dm²"""
        
        if unit_abbreviation == AreaUnits.SquareCentimeter:
            return """cm²"""
        
        if unit_abbreviation == AreaUnits.SquareMillimeter:
            return """mm²"""
        
        if unit_abbreviation == AreaUnits.SquareMicrometer:
            return """µm²"""
        
        if unit_abbreviation == AreaUnits.SquareMile:
            return """mi²"""
        
        if unit_abbreviation == AreaUnits.SquareYard:
            return """yd²"""
        
        if unit_abbreviation == AreaUnits.SquareFoot:
            return """ft²"""
        
        if unit_abbreviation == AreaUnits.UsSurveySquareFoot:
            return """ft² (US)"""
        
        if unit_abbreviation == AreaUnits.SquareInch:
            return """in²"""
        
        if unit_abbreviation == AreaUnits.Acre:
            return """ac"""
        
        if unit_abbreviation == AreaUnits.Hectare:
            return """ha"""
        
        if unit_abbreviation == AreaUnits.SquareNauticalMile:
            return """nmi²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RotationalStiffnessUnits(Enum):
        """
            RotationalStiffnessUnits enumeration
        """
        
        NewtonMeterPerRadian = 'NewtonMeterPerRadian'
        """
            
        """
        
        PoundForceFootPerDegrees = 'PoundForceFootPerDegrees'
        """
            
        """
        
        KilopoundForceFootPerDegrees = 'KilopoundForceFootPerDegrees'
        """
            
        """
        
        NewtonMillimeterPerDegree = 'NewtonMillimeterPerDegree'
        """
            
        """
        
        NewtonMeterPerDegree = 'NewtonMeterPerDegree'
        """
            
        """
        
        NewtonMillimeterPerRadian = 'NewtonMillimeterPerRadian'
        """
            
        """
        
        PoundForceFeetPerRadian = 'PoundForceFeetPerRadian'
        """
            
        """
        
        KilonewtonMeterPerRadian = 'KilonewtonMeterPerRadian'
        """
            
        """
        
        MeganewtonMeterPerRadian = 'MeganewtonMeterPerRadian'
        """
            
        """
        
        NanonewtonMillimeterPerDegree = 'NanonewtonMillimeterPerDegree'
        """
            
        """
        
        MicronewtonMillimeterPerDegree = 'MicronewtonMillimeterPerDegree'
        """
            
        """
        
        MillinewtonMillimeterPerDegree = 'MillinewtonMillimeterPerDegree'
        """
            
        """
        
        CentinewtonMillimeterPerDegree = 'CentinewtonMillimeterPerDegree'
        """
            
        """
        
        DecinewtonMillimeterPerDegree = 'DecinewtonMillimeterPerDegree'
        """
            
        """
        
        DecanewtonMillimeterPerDegree = 'DecanewtonMillimeterPerDegree'
        """
            
        """
        
        KilonewtonMillimeterPerDegree = 'KilonewtonMillimeterPerDegree'
        """
            
        """
        
        MeganewtonMillimeterPerDegree = 'MeganewtonMillimeterPerDegree'
        """
            
        """
        
        NanonewtonMeterPerDegree = 'NanonewtonMeterPerDegree'
        """
            
        """
        
        MicronewtonMeterPerDegree = 'MicronewtonMeterPerDegree'
        """
            
        """
        
        MillinewtonMeterPerDegree = 'MillinewtonMeterPerDegree'
        """
            
        """
        
        CentinewtonMeterPerDegree = 'CentinewtonMeterPerDegree'
        """
            
        """
        
        DecinewtonMeterPerDegree = 'DecinewtonMeterPerDegree'
        """
            
        """
        
        DecanewtonMeterPerDegree = 'DecanewtonMeterPerDegree'
        """
            
        """
        
        KilonewtonMeterPerDegree = 'KilonewtonMeterPerDegree'
        """
            
        """
        
        MeganewtonMeterPerDegree = 'MeganewtonMeterPerDegree'
        """
            
        """
        
        NanonewtonMillimeterPerRadian = 'NanonewtonMillimeterPerRadian'
        """
            
        """
        
        MicronewtonMillimeterPerRadian = 'MicronewtonMillimeterPerRadian'
        """
            
        """
        
        MillinewtonMillimeterPerRadian = 'MillinewtonMillimeterPerRadian'
        """
            
        """
        
        CentinewtonMillimeterPerRadian = 'CentinewtonMillimeterPerRadian'
        """
            
        """
        
        DecinewtonMillimeterPerRadian = 'DecinewtonMillimeterPerRadian'
        """
            
        """
        
        DecanewtonMillimeterPerRadian = 'DecanewtonMillimeterPerRadian'
        """
            
        """
        
        KilonewtonMillimeterPerRadian = 'KilonewtonMillimeterPerRadian'
        """
            
        """
        
        MeganewtonMillimeterPerRadian = 'MeganewtonMillimeterPerRadian'
        """
            
        """
        

class RotationalStiffnessDto:
    """
    A DTO representation of a RotationalStiffness

    Attributes:
        value (float): The value of the RotationalStiffness.
        unit (RotationalStiffnessUnits): The specific unit that the RotationalStiffness value is representing.
    """

    def __init__(self, value: float, unit: RotationalStiffnessUnits):
        """
        Create a new DTO representation of a RotationalStiffness

        Parameters:
            value (float): The value of the RotationalStiffness.
            unit (RotationalStiffnessUnits): The specific unit that the RotationalStiffness value is representing.
        """
        self.value: float = value
        """
        The value of the RotationalStiffness
        """
        self.unit: RotationalStiffnessUnits = unit
        """
        The specific unit that the RotationalStiffness value is representing
        """

    def to_json(self):
        """
        Get a RotationalStiffness DTO JSON object representing the current unit.

        :return: JSON object represents RotationalStiffness DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonMeterPerRadian"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RotationalStiffness DTO from a json representation.

        :param data: The RotationalStiffness DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonMeterPerRadian"}
        :return: A new instance of RotationalStiffnessDto.
        :rtype: RotationalStiffnessDto
        """
        return RotationalStiffnessDto(value=data["value"], unit=RotationalStiffnessUnits(data["unit"]))


class RotationalStiffness(AbstractMeasure):
    """
    https://en.wikipedia.org/wiki/Stiffness#Rotational_stiffness

    Args:
        value (float): The value.
        from_unit (RotationalStiffnessUnits): The RotationalStiffness unit to create from, The default unit is NewtonMeterPerRadian
    """
    def __init__(self, value: float, from_unit: RotationalStiffnessUnits = RotationalStiffnessUnits.NewtonMeterPerRadian):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__newton_meters_per_radian = None
        
        self.__pound_force_feet_per_degrees = None
        
        self.__kilopound_force_feet_per_degrees = None
        
        self.__newton_millimeters_per_degree = None
        
        self.__newton_meters_per_degree = None
        
        self.__newton_millimeters_per_radian = None
        
        self.__pound_force_feet_per_radian = None
        
        self.__kilonewton_meters_per_radian = None
        
        self.__meganewton_meters_per_radian = None
        
        self.__nanonewton_millimeters_per_degree = None
        
        self.__micronewton_millimeters_per_degree = None
        
        self.__millinewton_millimeters_per_degree = None
        
        self.__centinewton_millimeters_per_degree = None
        
        self.__decinewton_millimeters_per_degree = None
        
        self.__decanewton_millimeters_per_degree = None
        
        self.__kilonewton_millimeters_per_degree = None
        
        self.__meganewton_millimeters_per_degree = None
        
        self.__nanonewton_meters_per_degree = None
        
        self.__micronewton_meters_per_degree = None
        
        self.__millinewton_meters_per_degree = None
        
        self.__centinewton_meters_per_degree = None
        
        self.__decinewton_meters_per_degree = None
        
        self.__decanewton_meters_per_degree = None
        
        self.__kilonewton_meters_per_degree = None
        
        self.__meganewton_meters_per_degree = None
        
        self.__nanonewton_millimeters_per_radian = None
        
        self.__micronewton_millimeters_per_radian = None
        
        self.__millinewton_millimeters_per_radian = None
        
        self.__centinewton_millimeters_per_radian = None
        
        self.__decinewton_millimeters_per_radian = None
        
        self.__decanewton_millimeters_per_radian = None
        
        self.__kilonewton_millimeters_per_radian = None
        
        self.__meganewton_millimeters_per_radian = None
        

    def convert(self, unit: RotationalStiffnessUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RotationalStiffnessUnits = RotationalStiffnessUnits.NewtonMeterPerRadian) -> RotationalStiffnessDto:
        """
        Get a new instance of RotationalStiffness DTO representing the current unit.

        :param hold_in_unit: The specific RotationalStiffness unit to store the RotationalStiffness value in the DTO representation.
        :type hold_in_unit: RotationalStiffnessUnits
        :return: A new instance of RotationalStiffnessDto.
        :rtype: RotationalStiffnessDto
        """
        return RotationalStiffnessDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RotationalStiffnessUnits = RotationalStiffnessUnits.NewtonMeterPerRadian):
        """
        Get a RotationalStiffness DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RotationalStiffness unit to store the RotationalStiffness value in the DTO representation.
        :type hold_in_unit: RotationalStiffnessUnits
        :return: JSON object represents RotationalStiffness DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonMeterPerRadian"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(rotational_stiffness_dto: RotationalStiffnessDto):
        """
        Obtain a new instance of RotationalStiffness from a DTO unit object.

        :param rotational_stiffness_dto: The RotationalStiffness DTO representation.
        :type rotational_stiffness_dto: RotationalStiffnessDto
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(rotational_stiffness_dto.value, rotational_stiffness_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RotationalStiffness from a DTO unit json representation.

        :param data: The RotationalStiffness DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonMeterPerRadian"}
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness.from_dto(RotationalStiffnessDto.from_json(data))

    def __convert_from_base(self, from_unit: RotationalStiffnessUnits) -> float:
        value = self._value
        
        if from_unit == RotationalStiffnessUnits.NewtonMeterPerRadian:
            return (value)
        
        if from_unit == RotationalStiffnessUnits.PoundForceFootPerDegrees:
            return (value / 77.6826)
        
        if from_unit == RotationalStiffnessUnits.KilopoundForceFootPerDegrees:
            return (value / 77682.6)
        
        if from_unit == RotationalStiffnessUnits.NewtonMillimeterPerDegree:
            return (value / 180 * math.pi * 1000)
        
        if from_unit == RotationalStiffnessUnits.NewtonMeterPerDegree:
            return (value / (180 / math.pi))
        
        if from_unit == RotationalStiffnessUnits.NewtonMillimeterPerRadian:
            return (value * 1000)
        
        if from_unit == RotationalStiffnessUnits.PoundForceFeetPerRadian:
            return (value / 1.3558179483314)
        
        if from_unit == RotationalStiffnessUnits.KilonewtonMeterPerRadian:
            return ((value) / 1000.0)
        
        if from_unit == RotationalStiffnessUnits.MeganewtonMeterPerRadian:
            return ((value) / 1000000.0)
        
        if from_unit == RotationalStiffnessUnits.NanonewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 1e-09)
        
        if from_unit == RotationalStiffnessUnits.MicronewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 1e-06)
        
        if from_unit == RotationalStiffnessUnits.MillinewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 0.001)
        
        if from_unit == RotationalStiffnessUnits.CentinewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 0.01)
        
        if from_unit == RotationalStiffnessUnits.DecinewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 0.1)
        
        if from_unit == RotationalStiffnessUnits.DecanewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 10.0)
        
        if from_unit == RotationalStiffnessUnits.KilonewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 1000.0)
        
        if from_unit == RotationalStiffnessUnits.MeganewtonMillimeterPerDegree:
            return ((value / 180 * math.pi * 1000) / 1000000.0)
        
        if from_unit == RotationalStiffnessUnits.NanonewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 1e-09)
        
        if from_unit == RotationalStiffnessUnits.MicronewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 1e-06)
        
        if from_unit == RotationalStiffnessUnits.MillinewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 0.001)
        
        if from_unit == RotationalStiffnessUnits.CentinewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 0.01)
        
        if from_unit == RotationalStiffnessUnits.DecinewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 0.1)
        
        if from_unit == RotationalStiffnessUnits.DecanewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 10.0)
        
        if from_unit == RotationalStiffnessUnits.KilonewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 1000.0)
        
        if from_unit == RotationalStiffnessUnits.MeganewtonMeterPerDegree:
            return ((value / (180 / math.pi)) / 1000000.0)
        
        if from_unit == RotationalStiffnessUnits.NanonewtonMillimeterPerRadian:
            return ((value * 1000) / 1e-09)
        
        if from_unit == RotationalStiffnessUnits.MicronewtonMillimeterPerRadian:
            return ((value * 1000) / 1e-06)
        
        if from_unit == RotationalStiffnessUnits.MillinewtonMillimeterPerRadian:
            return ((value * 1000) / 0.001)
        
        if from_unit == RotationalStiffnessUnits.CentinewtonMillimeterPerRadian:
            return ((value * 1000) / 0.01)
        
        if from_unit == RotationalStiffnessUnits.DecinewtonMillimeterPerRadian:
            return ((value * 1000) / 0.1)
        
        if from_unit == RotationalStiffnessUnits.DecanewtonMillimeterPerRadian:
            return ((value * 1000) / 10.0)
        
        if from_unit == RotationalStiffnessUnits.KilonewtonMillimeterPerRadian:
            return ((value * 1000) / 1000.0)
        
        if from_unit == RotationalStiffnessUnits.MeganewtonMillimeterPerRadian:
            return ((value * 1000) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RotationalStiffnessUnits) -> float:
        
        if to_unit == RotationalStiffnessUnits.NewtonMeterPerRadian:
            return (value)
        
        if to_unit == RotationalStiffnessUnits.PoundForceFootPerDegrees:
            return (value * 77.6826)
        
        if to_unit == RotationalStiffnessUnits.KilopoundForceFootPerDegrees:
            return (value * 77682.6)
        
        if to_unit == RotationalStiffnessUnits.NewtonMillimeterPerDegree:
            return (value * 180 / math.pi * 0.001)
        
        if to_unit == RotationalStiffnessUnits.NewtonMeterPerDegree:
            return (value * (180 / math.pi))
        
        if to_unit == RotationalStiffnessUnits.NewtonMillimeterPerRadian:
            return (value * 0.001)
        
        if to_unit == RotationalStiffnessUnits.PoundForceFeetPerRadian:
            return (value * 1.3558179483314)
        
        if to_unit == RotationalStiffnessUnits.KilonewtonMeterPerRadian:
            return ((value) * 1000.0)
        
        if to_unit == RotationalStiffnessUnits.MeganewtonMeterPerRadian:
            return ((value) * 1000000.0)
        
        if to_unit == RotationalStiffnessUnits.NanonewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 1e-09)
        
        if to_unit == RotationalStiffnessUnits.MicronewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 1e-06)
        
        if to_unit == RotationalStiffnessUnits.MillinewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 0.001)
        
        if to_unit == RotationalStiffnessUnits.CentinewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 0.01)
        
        if to_unit == RotationalStiffnessUnits.DecinewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 0.1)
        
        if to_unit == RotationalStiffnessUnits.DecanewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 10.0)
        
        if to_unit == RotationalStiffnessUnits.KilonewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 1000.0)
        
        if to_unit == RotationalStiffnessUnits.MeganewtonMillimeterPerDegree:
            return ((value * 180 / math.pi * 0.001) * 1000000.0)
        
        if to_unit == RotationalStiffnessUnits.NanonewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 1e-09)
        
        if to_unit == RotationalStiffnessUnits.MicronewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 1e-06)
        
        if to_unit == RotationalStiffnessUnits.MillinewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 0.001)
        
        if to_unit == RotationalStiffnessUnits.CentinewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 0.01)
        
        if to_unit == RotationalStiffnessUnits.DecinewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 0.1)
        
        if to_unit == RotationalStiffnessUnits.DecanewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 10.0)
        
        if to_unit == RotationalStiffnessUnits.KilonewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 1000.0)
        
        if to_unit == RotationalStiffnessUnits.MeganewtonMeterPerDegree:
            return ((value * (180 / math.pi)) * 1000000.0)
        
        if to_unit == RotationalStiffnessUnits.NanonewtonMillimeterPerRadian:
            return ((value * 0.001) * 1e-09)
        
        if to_unit == RotationalStiffnessUnits.MicronewtonMillimeterPerRadian:
            return ((value * 0.001) * 1e-06)
        
        if to_unit == RotationalStiffnessUnits.MillinewtonMillimeterPerRadian:
            return ((value * 0.001) * 0.001)
        
        if to_unit == RotationalStiffnessUnits.CentinewtonMillimeterPerRadian:
            return ((value * 0.001) * 0.01)
        
        if to_unit == RotationalStiffnessUnits.DecinewtonMillimeterPerRadian:
            return ((value * 0.001) * 0.1)
        
        if to_unit == RotationalStiffnessUnits.DecanewtonMillimeterPerRadian:
            return ((value * 0.001) * 10.0)
        
        if to_unit == RotationalStiffnessUnits.KilonewtonMillimeterPerRadian:
            return ((value * 0.001) * 1000.0)
        
        if to_unit == RotationalStiffnessUnits.MeganewtonMillimeterPerRadian:
            return ((value * 0.001) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_newton_meters_per_radian(newton_meters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in newton_meters_per_radian.

        

        :param meters: The RotationalStiffness value in newton_meters_per_radian.
        :type newton_meters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(newton_meters_per_radian, RotationalStiffnessUnits.NewtonMeterPerRadian)

    
    @staticmethod
    def from_pound_force_feet_per_degrees(pound_force_feet_per_degrees: float):
        """
        Create a new instance of RotationalStiffness from a value in pound_force_feet_per_degrees.

        

        :param meters: The RotationalStiffness value in pound_force_feet_per_degrees.
        :type pound_force_feet_per_degrees: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(pound_force_feet_per_degrees, RotationalStiffnessUnits.PoundForceFootPerDegrees)

    
    @staticmethod
    def from_kilopound_force_feet_per_degrees(kilopound_force_feet_per_degrees: float):
        """
        Create a new instance of RotationalStiffness from a value in kilopound_force_feet_per_degrees.

        

        :param meters: The RotationalStiffness value in kilopound_force_feet_per_degrees.
        :type kilopound_force_feet_per_degrees: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(kilopound_force_feet_per_degrees, RotationalStiffnessUnits.KilopoundForceFootPerDegrees)

    
    @staticmethod
    def from_newton_millimeters_per_degree(newton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in newton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in newton_millimeters_per_degree.
        :type newton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(newton_millimeters_per_degree, RotationalStiffnessUnits.NewtonMillimeterPerDegree)

    
    @staticmethod
    def from_newton_meters_per_degree(newton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in newton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in newton_meters_per_degree.
        :type newton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(newton_meters_per_degree, RotationalStiffnessUnits.NewtonMeterPerDegree)

    
    @staticmethod
    def from_newton_millimeters_per_radian(newton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in newton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in newton_millimeters_per_radian.
        :type newton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(newton_millimeters_per_radian, RotationalStiffnessUnits.NewtonMillimeterPerRadian)

    
    @staticmethod
    def from_pound_force_feet_per_radian(pound_force_feet_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in pound_force_feet_per_radian.

        

        :param meters: The RotationalStiffness value in pound_force_feet_per_radian.
        :type pound_force_feet_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(pound_force_feet_per_radian, RotationalStiffnessUnits.PoundForceFeetPerRadian)

    
    @staticmethod
    def from_kilonewton_meters_per_radian(kilonewton_meters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in kilonewton_meters_per_radian.

        

        :param meters: The RotationalStiffness value in kilonewton_meters_per_radian.
        :type kilonewton_meters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(kilonewton_meters_per_radian, RotationalStiffnessUnits.KilonewtonMeterPerRadian)

    
    @staticmethod
    def from_meganewton_meters_per_radian(meganewton_meters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in meganewton_meters_per_radian.

        

        :param meters: The RotationalStiffness value in meganewton_meters_per_radian.
        :type meganewton_meters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(meganewton_meters_per_radian, RotationalStiffnessUnits.MeganewtonMeterPerRadian)

    
    @staticmethod
    def from_nanonewton_millimeters_per_degree(nanonewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in nanonewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in nanonewton_millimeters_per_degree.
        :type nanonewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(nanonewton_millimeters_per_degree, RotationalStiffnessUnits.NanonewtonMillimeterPerDegree)

    
    @staticmethod
    def from_micronewton_millimeters_per_degree(micronewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in micronewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in micronewton_millimeters_per_degree.
        :type micronewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(micronewton_millimeters_per_degree, RotationalStiffnessUnits.MicronewtonMillimeterPerDegree)

    
    @staticmethod
    def from_millinewton_millimeters_per_degree(millinewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in millinewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in millinewton_millimeters_per_degree.
        :type millinewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(millinewton_millimeters_per_degree, RotationalStiffnessUnits.MillinewtonMillimeterPerDegree)

    
    @staticmethod
    def from_centinewton_millimeters_per_degree(centinewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in centinewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in centinewton_millimeters_per_degree.
        :type centinewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(centinewton_millimeters_per_degree, RotationalStiffnessUnits.CentinewtonMillimeterPerDegree)

    
    @staticmethod
    def from_decinewton_millimeters_per_degree(decinewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in decinewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in decinewton_millimeters_per_degree.
        :type decinewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(decinewton_millimeters_per_degree, RotationalStiffnessUnits.DecinewtonMillimeterPerDegree)

    
    @staticmethod
    def from_decanewton_millimeters_per_degree(decanewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in decanewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in decanewton_millimeters_per_degree.
        :type decanewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(decanewton_millimeters_per_degree, RotationalStiffnessUnits.DecanewtonMillimeterPerDegree)

    
    @staticmethod
    def from_kilonewton_millimeters_per_degree(kilonewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in kilonewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in kilonewton_millimeters_per_degree.
        :type kilonewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(kilonewton_millimeters_per_degree, RotationalStiffnessUnits.KilonewtonMillimeterPerDegree)

    
    @staticmethod
    def from_meganewton_millimeters_per_degree(meganewton_millimeters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in meganewton_millimeters_per_degree.

        

        :param meters: The RotationalStiffness value in meganewton_millimeters_per_degree.
        :type meganewton_millimeters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(meganewton_millimeters_per_degree, RotationalStiffnessUnits.MeganewtonMillimeterPerDegree)

    
    @staticmethod
    def from_nanonewton_meters_per_degree(nanonewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in nanonewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in nanonewton_meters_per_degree.
        :type nanonewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(nanonewton_meters_per_degree, RotationalStiffnessUnits.NanonewtonMeterPerDegree)

    
    @staticmethod
    def from_micronewton_meters_per_degree(micronewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in micronewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in micronewton_meters_per_degree.
        :type micronewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(micronewton_meters_per_degree, RotationalStiffnessUnits.MicronewtonMeterPerDegree)

    
    @staticmethod
    def from_millinewton_meters_per_degree(millinewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in millinewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in millinewton_meters_per_degree.
        :type millinewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(millinewton_meters_per_degree, RotationalStiffnessUnits.MillinewtonMeterPerDegree)

    
    @staticmethod
    def from_centinewton_meters_per_degree(centinewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in centinewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in centinewton_meters_per_degree.
        :type centinewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(centinewton_meters_per_degree, RotationalStiffnessUnits.CentinewtonMeterPerDegree)

    
    @staticmethod
    def from_decinewton_meters_per_degree(decinewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in decinewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in decinewton_meters_per_degree.
        :type decinewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(decinewton_meters_per_degree, RotationalStiffnessUnits.DecinewtonMeterPerDegree)

    
    @staticmethod
    def from_decanewton_meters_per_degree(decanewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in decanewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in decanewton_meters_per_degree.
        :type decanewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(decanewton_meters_per_degree, RotationalStiffnessUnits.DecanewtonMeterPerDegree)

    
    @staticmethod
    def from_kilonewton_meters_per_degree(kilonewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in kilonewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in kilonewton_meters_per_degree.
        :type kilonewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(kilonewton_meters_per_degree, RotationalStiffnessUnits.KilonewtonMeterPerDegree)

    
    @staticmethod
    def from_meganewton_meters_per_degree(meganewton_meters_per_degree: float):
        """
        Create a new instance of RotationalStiffness from a value in meganewton_meters_per_degree.

        

        :param meters: The RotationalStiffness value in meganewton_meters_per_degree.
        :type meganewton_meters_per_degree: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(meganewton_meters_per_degree, RotationalStiffnessUnits.MeganewtonMeterPerDegree)

    
    @staticmethod
    def from_nanonewton_millimeters_per_radian(nanonewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in nanonewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in nanonewton_millimeters_per_radian.
        :type nanonewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(nanonewton_millimeters_per_radian, RotationalStiffnessUnits.NanonewtonMillimeterPerRadian)

    
    @staticmethod
    def from_micronewton_millimeters_per_radian(micronewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in micronewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in micronewton_millimeters_per_radian.
        :type micronewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(micronewton_millimeters_per_radian, RotationalStiffnessUnits.MicronewtonMillimeterPerRadian)

    
    @staticmethod
    def from_millinewton_millimeters_per_radian(millinewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in millinewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in millinewton_millimeters_per_radian.
        :type millinewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(millinewton_millimeters_per_radian, RotationalStiffnessUnits.MillinewtonMillimeterPerRadian)

    
    @staticmethod
    def from_centinewton_millimeters_per_radian(centinewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in centinewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in centinewton_millimeters_per_radian.
        :type centinewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(centinewton_millimeters_per_radian, RotationalStiffnessUnits.CentinewtonMillimeterPerRadian)

    
    @staticmethod
    def from_decinewton_millimeters_per_radian(decinewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in decinewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in decinewton_millimeters_per_radian.
        :type decinewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(decinewton_millimeters_per_radian, RotationalStiffnessUnits.DecinewtonMillimeterPerRadian)

    
    @staticmethod
    def from_decanewton_millimeters_per_radian(decanewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in decanewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in decanewton_millimeters_per_radian.
        :type decanewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(decanewton_millimeters_per_radian, RotationalStiffnessUnits.DecanewtonMillimeterPerRadian)

    
    @staticmethod
    def from_kilonewton_millimeters_per_radian(kilonewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in kilonewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in kilonewton_millimeters_per_radian.
        :type kilonewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(kilonewton_millimeters_per_radian, RotationalStiffnessUnits.KilonewtonMillimeterPerRadian)

    
    @staticmethod
    def from_meganewton_millimeters_per_radian(meganewton_millimeters_per_radian: float):
        """
        Create a new instance of RotationalStiffness from a value in meganewton_millimeters_per_radian.

        

        :param meters: The RotationalStiffness value in meganewton_millimeters_per_radian.
        :type meganewton_millimeters_per_radian: float
        :return: A new instance of RotationalStiffness.
        :rtype: RotationalStiffness
        """
        return RotationalStiffness(meganewton_millimeters_per_radian, RotationalStiffnessUnits.MeganewtonMillimeterPerRadian)

    
    @property
    def newton_meters_per_radian(self) -> float:
        """
        
        """
        if self.__newton_meters_per_radian != None:
            return self.__newton_meters_per_radian
        self.__newton_meters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.NewtonMeterPerRadian)
        return self.__newton_meters_per_radian

    
    @property
    def pound_force_feet_per_degrees(self) -> float:
        """
        
        """
        if self.__pound_force_feet_per_degrees != None:
            return self.__pound_force_feet_per_degrees
        self.__pound_force_feet_per_degrees = self.__convert_from_base(RotationalStiffnessUnits.PoundForceFootPerDegrees)
        return self.__pound_force_feet_per_degrees

    
    @property
    def kilopound_force_feet_per_degrees(self) -> float:
        """
        
        """
        if self.__kilopound_force_feet_per_degrees != None:
            return self.__kilopound_force_feet_per_degrees
        self.__kilopound_force_feet_per_degrees = self.__convert_from_base(RotationalStiffnessUnits.KilopoundForceFootPerDegrees)
        return self.__kilopound_force_feet_per_degrees

    
    @property
    def newton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__newton_millimeters_per_degree != None:
            return self.__newton_millimeters_per_degree
        self.__newton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.NewtonMillimeterPerDegree)
        return self.__newton_millimeters_per_degree

    
    @property
    def newton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__newton_meters_per_degree != None:
            return self.__newton_meters_per_degree
        self.__newton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.NewtonMeterPerDegree)
        return self.__newton_meters_per_degree

    
    @property
    def newton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__newton_millimeters_per_radian != None:
            return self.__newton_millimeters_per_radian
        self.__newton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.NewtonMillimeterPerRadian)
        return self.__newton_millimeters_per_radian

    
    @property
    def pound_force_feet_per_radian(self) -> float:
        """
        
        """
        if self.__pound_force_feet_per_radian != None:
            return self.__pound_force_feet_per_radian
        self.__pound_force_feet_per_radian = self.__convert_from_base(RotationalStiffnessUnits.PoundForceFeetPerRadian)
        return self.__pound_force_feet_per_radian

    
    @property
    def kilonewton_meters_per_radian(self) -> float:
        """
        
        """
        if self.__kilonewton_meters_per_radian != None:
            return self.__kilonewton_meters_per_radian
        self.__kilonewton_meters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.KilonewtonMeterPerRadian)
        return self.__kilonewton_meters_per_radian

    
    @property
    def meganewton_meters_per_radian(self) -> float:
        """
        
        """
        if self.__meganewton_meters_per_radian != None:
            return self.__meganewton_meters_per_radian
        self.__meganewton_meters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.MeganewtonMeterPerRadian)
        return self.__meganewton_meters_per_radian

    
    @property
    def nanonewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__nanonewton_millimeters_per_degree != None:
            return self.__nanonewton_millimeters_per_degree
        self.__nanonewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.NanonewtonMillimeterPerDegree)
        return self.__nanonewton_millimeters_per_degree

    
    @property
    def micronewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__micronewton_millimeters_per_degree != None:
            return self.__micronewton_millimeters_per_degree
        self.__micronewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.MicronewtonMillimeterPerDegree)
        return self.__micronewton_millimeters_per_degree

    
    @property
    def millinewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__millinewton_millimeters_per_degree != None:
            return self.__millinewton_millimeters_per_degree
        self.__millinewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.MillinewtonMillimeterPerDegree)
        return self.__millinewton_millimeters_per_degree

    
    @property
    def centinewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__centinewton_millimeters_per_degree != None:
            return self.__centinewton_millimeters_per_degree
        self.__centinewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.CentinewtonMillimeterPerDegree)
        return self.__centinewton_millimeters_per_degree

    
    @property
    def decinewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__decinewton_millimeters_per_degree != None:
            return self.__decinewton_millimeters_per_degree
        self.__decinewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.DecinewtonMillimeterPerDegree)
        return self.__decinewton_millimeters_per_degree

    
    @property
    def decanewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__decanewton_millimeters_per_degree != None:
            return self.__decanewton_millimeters_per_degree
        self.__decanewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.DecanewtonMillimeterPerDegree)
        return self.__decanewton_millimeters_per_degree

    
    @property
    def kilonewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__kilonewton_millimeters_per_degree != None:
            return self.__kilonewton_millimeters_per_degree
        self.__kilonewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.KilonewtonMillimeterPerDegree)
        return self.__kilonewton_millimeters_per_degree

    
    @property
    def meganewton_millimeters_per_degree(self) -> float:
        """
        
        """
        if self.__meganewton_millimeters_per_degree != None:
            return self.__meganewton_millimeters_per_degree
        self.__meganewton_millimeters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.MeganewtonMillimeterPerDegree)
        return self.__meganewton_millimeters_per_degree

    
    @property
    def nanonewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__nanonewton_meters_per_degree != None:
            return self.__nanonewton_meters_per_degree
        self.__nanonewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.NanonewtonMeterPerDegree)
        return self.__nanonewton_meters_per_degree

    
    @property
    def micronewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__micronewton_meters_per_degree != None:
            return self.__micronewton_meters_per_degree
        self.__micronewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.MicronewtonMeterPerDegree)
        return self.__micronewton_meters_per_degree

    
    @property
    def millinewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__millinewton_meters_per_degree != None:
            return self.__millinewton_meters_per_degree
        self.__millinewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.MillinewtonMeterPerDegree)
        return self.__millinewton_meters_per_degree

    
    @property
    def centinewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__centinewton_meters_per_degree != None:
            return self.__centinewton_meters_per_degree
        self.__centinewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.CentinewtonMeterPerDegree)
        return self.__centinewton_meters_per_degree

    
    @property
    def decinewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__decinewton_meters_per_degree != None:
            return self.__decinewton_meters_per_degree
        self.__decinewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.DecinewtonMeterPerDegree)
        return self.__decinewton_meters_per_degree

    
    @property
    def decanewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__decanewton_meters_per_degree != None:
            return self.__decanewton_meters_per_degree
        self.__decanewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.DecanewtonMeterPerDegree)
        return self.__decanewton_meters_per_degree

    
    @property
    def kilonewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__kilonewton_meters_per_degree != None:
            return self.__kilonewton_meters_per_degree
        self.__kilonewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.KilonewtonMeterPerDegree)
        return self.__kilonewton_meters_per_degree

    
    @property
    def meganewton_meters_per_degree(self) -> float:
        """
        
        """
        if self.__meganewton_meters_per_degree != None:
            return self.__meganewton_meters_per_degree
        self.__meganewton_meters_per_degree = self.__convert_from_base(RotationalStiffnessUnits.MeganewtonMeterPerDegree)
        return self.__meganewton_meters_per_degree

    
    @property
    def nanonewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__nanonewton_millimeters_per_radian != None:
            return self.__nanonewton_millimeters_per_radian
        self.__nanonewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.NanonewtonMillimeterPerRadian)
        return self.__nanonewton_millimeters_per_radian

    
    @property
    def micronewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__micronewton_millimeters_per_radian != None:
            return self.__micronewton_millimeters_per_radian
        self.__micronewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.MicronewtonMillimeterPerRadian)
        return self.__micronewton_millimeters_per_radian

    
    @property
    def millinewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__millinewton_millimeters_per_radian != None:
            return self.__millinewton_millimeters_per_radian
        self.__millinewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.MillinewtonMillimeterPerRadian)
        return self.__millinewton_millimeters_per_radian

    
    @property
    def centinewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__centinewton_millimeters_per_radian != None:
            return self.__centinewton_millimeters_per_radian
        self.__centinewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.CentinewtonMillimeterPerRadian)
        return self.__centinewton_millimeters_per_radian

    
    @property
    def decinewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__decinewton_millimeters_per_radian != None:
            return self.__decinewton_millimeters_per_radian
        self.__decinewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.DecinewtonMillimeterPerRadian)
        return self.__decinewton_millimeters_per_radian

    
    @property
    def decanewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__decanewton_millimeters_per_radian != None:
            return self.__decanewton_millimeters_per_radian
        self.__decanewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.DecanewtonMillimeterPerRadian)
        return self.__decanewton_millimeters_per_radian

    
    @property
    def kilonewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__kilonewton_millimeters_per_radian != None:
            return self.__kilonewton_millimeters_per_radian
        self.__kilonewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.KilonewtonMillimeterPerRadian)
        return self.__kilonewton_millimeters_per_radian

    
    @property
    def meganewton_millimeters_per_radian(self) -> float:
        """
        
        """
        if self.__meganewton_millimeters_per_radian != None:
            return self.__meganewton_millimeters_per_radian
        self.__meganewton_millimeters_per_radian = self.__convert_from_base(RotationalStiffnessUnits.MeganewtonMillimeterPerRadian)
        return self.__meganewton_millimeters_per_radian

    
    def to_string(self, unit: RotationalStiffnessUnits = RotationalStiffnessUnits.NewtonMeterPerRadian, fractional_digits: int = None) -> str:
        """
        Format the RotationalStiffness to a string.
        
        Note: the default format for RotationalStiffness is NewtonMeterPerRadian.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RotationalStiffness. Default is 'NewtonMeterPerRadian'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RotationalStiffnessUnits.NewtonMeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.newton_meters_per_radian, fractional_digits)} N·m/rad"""
        
        if unit == RotationalStiffnessUnits.PoundForceFootPerDegrees:
            return f"""{super()._truncate_fraction_digits(self.pound_force_feet_per_degrees, fractional_digits)} lbf·ft/deg"""
        
        if unit == RotationalStiffnessUnits.KilopoundForceFootPerDegrees:
            return f"""{super()._truncate_fraction_digits(self.kilopound_force_feet_per_degrees, fractional_digits)} kipf·ft/°"""
        
        if unit == RotationalStiffnessUnits.NewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.newton_millimeters_per_degree, fractional_digits)} N·mm/deg"""
        
        if unit == RotationalStiffnessUnits.NewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.newton_meters_per_degree, fractional_digits)} N·m/deg"""
        
        if unit == RotationalStiffnessUnits.NewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.newton_millimeters_per_radian, fractional_digits)} N·mm/rad"""
        
        if unit == RotationalStiffnessUnits.PoundForceFeetPerRadian:
            return f"""{super()._truncate_fraction_digits(self.pound_force_feet_per_radian, fractional_digits)} lbf·ft/rad"""
        
        if unit == RotationalStiffnessUnits.KilonewtonMeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_meters_per_radian, fractional_digits)} kN·m/rad"""
        
        if unit == RotationalStiffnessUnits.MeganewtonMeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.meganewton_meters_per_radian, fractional_digits)} MN·m/rad"""
        
        if unit == RotationalStiffnessUnits.NanonewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.nanonewton_millimeters_per_degree, fractional_digits)} nN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.MicronewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.micronewton_millimeters_per_degree, fractional_digits)} μN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.MillinewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.millinewton_millimeters_per_degree, fractional_digits)} mN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.CentinewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.centinewton_millimeters_per_degree, fractional_digits)} cN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.DecinewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.decinewton_millimeters_per_degree, fractional_digits)} dN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.DecanewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.decanewton_millimeters_per_degree, fractional_digits)} daN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.KilonewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_millimeters_per_degree, fractional_digits)} kN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.MeganewtonMillimeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.meganewton_millimeters_per_degree, fractional_digits)} MN·mm/deg"""
        
        if unit == RotationalStiffnessUnits.NanonewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.nanonewton_meters_per_degree, fractional_digits)} nN·m/deg"""
        
        if unit == RotationalStiffnessUnits.MicronewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.micronewton_meters_per_degree, fractional_digits)} μN·m/deg"""
        
        if unit == RotationalStiffnessUnits.MillinewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.millinewton_meters_per_degree, fractional_digits)} mN·m/deg"""
        
        if unit == RotationalStiffnessUnits.CentinewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.centinewton_meters_per_degree, fractional_digits)} cN·m/deg"""
        
        if unit == RotationalStiffnessUnits.DecinewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.decinewton_meters_per_degree, fractional_digits)} dN·m/deg"""
        
        if unit == RotationalStiffnessUnits.DecanewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.decanewton_meters_per_degree, fractional_digits)} daN·m/deg"""
        
        if unit == RotationalStiffnessUnits.KilonewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_meters_per_degree, fractional_digits)} kN·m/deg"""
        
        if unit == RotationalStiffnessUnits.MeganewtonMeterPerDegree:
            return f"""{super()._truncate_fraction_digits(self.meganewton_meters_per_degree, fractional_digits)} MN·m/deg"""
        
        if unit == RotationalStiffnessUnits.NanonewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.nanonewton_millimeters_per_radian, fractional_digits)} nN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.MicronewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.micronewton_millimeters_per_radian, fractional_digits)} μN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.MillinewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.millinewton_millimeters_per_radian, fractional_digits)} mN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.CentinewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.centinewton_millimeters_per_radian, fractional_digits)} cN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.DecinewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.decinewton_millimeters_per_radian, fractional_digits)} dN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.DecanewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.decanewton_millimeters_per_radian, fractional_digits)} daN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.KilonewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_millimeters_per_radian, fractional_digits)} kN·mm/rad"""
        
        if unit == RotationalStiffnessUnits.MeganewtonMillimeterPerRadian:
            return f"""{super()._truncate_fraction_digits(self.meganewton_millimeters_per_radian, fractional_digits)} MN·mm/rad"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RotationalStiffnessUnits = RotationalStiffnessUnits.NewtonMeterPerRadian) -> str:
        """
        Get RotationalStiffness unit abbreviation.
        Note! the default abbreviation for RotationalStiffness is NewtonMeterPerRadian.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RotationalStiffnessUnits.NewtonMeterPerRadian:
            return """N·m/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.PoundForceFootPerDegrees:
            return """lbf·ft/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.KilopoundForceFootPerDegrees:
            return """kipf·ft/°"""
        
        if unit_abbreviation == RotationalStiffnessUnits.NewtonMillimeterPerDegree:
            return """N·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.NewtonMeterPerDegree:
            return """N·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.NewtonMillimeterPerRadian:
            return """N·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.PoundForceFeetPerRadian:
            return """lbf·ft/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.KilonewtonMeterPerRadian:
            return """kN·m/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MeganewtonMeterPerRadian:
            return """MN·m/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.NanonewtonMillimeterPerDegree:
            return """nN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MicronewtonMillimeterPerDegree:
            return """μN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MillinewtonMillimeterPerDegree:
            return """mN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.CentinewtonMillimeterPerDegree:
            return """cN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.DecinewtonMillimeterPerDegree:
            return """dN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.DecanewtonMillimeterPerDegree:
            return """daN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.KilonewtonMillimeterPerDegree:
            return """kN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MeganewtonMillimeterPerDegree:
            return """MN·mm/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.NanonewtonMeterPerDegree:
            return """nN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MicronewtonMeterPerDegree:
            return """μN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MillinewtonMeterPerDegree:
            return """mN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.CentinewtonMeterPerDegree:
            return """cN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.DecinewtonMeterPerDegree:
            return """dN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.DecanewtonMeterPerDegree:
            return """daN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.KilonewtonMeterPerDegree:
            return """kN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MeganewtonMeterPerDegree:
            return """MN·m/deg"""
        
        if unit_abbreviation == RotationalStiffnessUnits.NanonewtonMillimeterPerRadian:
            return """nN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MicronewtonMillimeterPerRadian:
            return """μN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MillinewtonMillimeterPerRadian:
            return """mN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.CentinewtonMillimeterPerRadian:
            return """cN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.DecinewtonMillimeterPerRadian:
            return """dN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.DecanewtonMillimeterPerRadian:
            return """daN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.KilonewtonMillimeterPerRadian:
            return """kN·mm/rad"""
        
        if unit_abbreviation == RotationalStiffnessUnits.MeganewtonMillimeterPerRadian:
            return """MN·mm/rad"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ImpulseUnits(Enum):
        """
            ImpulseUnits enumeration
        """
        
        KilogramMeterPerSecond = 'KilogramMeterPerSecond'
        """
            
        """
        
        NewtonSecond = 'NewtonSecond'
        """
            
        """
        
        PoundFootPerSecond = 'PoundFootPerSecond'
        """
            
        """
        
        PoundForceSecond = 'PoundForceSecond'
        """
            
        """
        
        SlugFootPerSecond = 'SlugFootPerSecond'
        """
            
        """
        
        NanonewtonSecond = 'NanonewtonSecond'
        """
            
        """
        
        MicronewtonSecond = 'MicronewtonSecond'
        """
            
        """
        
        MillinewtonSecond = 'MillinewtonSecond'
        """
            
        """
        
        CentinewtonSecond = 'CentinewtonSecond'
        """
            
        """
        
        DecinewtonSecond = 'DecinewtonSecond'
        """
            
        """
        
        DecanewtonSecond = 'DecanewtonSecond'
        """
            
        """
        
        KilonewtonSecond = 'KilonewtonSecond'
        """
            
        """
        
        MeganewtonSecond = 'MeganewtonSecond'
        """
            
        """
        

class ImpulseDto:
    """
    A DTO representation of a Impulse

    Attributes:
        value (float): The value of the Impulse.
        unit (ImpulseUnits): The specific unit that the Impulse value is representing.
    """

    def __init__(self, value: float, unit: ImpulseUnits):
        """
        Create a new DTO representation of a Impulse

        Parameters:
            value (float): The value of the Impulse.
            unit (ImpulseUnits): The specific unit that the Impulse value is representing.
        """
        self.value: float = value
        """
        The value of the Impulse
        """
        self.unit: ImpulseUnits = unit
        """
        The specific unit that the Impulse value is representing
        """

    def to_json(self):
        """
        Get a Impulse DTO JSON object representing the current unit.

        :return: JSON object represents Impulse DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Impulse DTO from a json representation.

        :param data: The Impulse DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonSecond"}
        :return: A new instance of ImpulseDto.
        :rtype: ImpulseDto
        """
        return ImpulseDto(value=data["value"], unit=ImpulseUnits(data["unit"]))


class Impulse(AbstractMeasure):
    """
    In classical mechanics, impulse is the integral of a force, F, over the time interval, t, for which it acts. Impulse applied to an object produces an equivalent vector change in its linear momentum, also in the resultant direction.

    Args:
        value (float): The value.
        from_unit (ImpulseUnits): The Impulse unit to create from, The default unit is NewtonSecond
    """
    def __init__(self, value: float, from_unit: ImpulseUnits = ImpulseUnits.NewtonSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__kilogram_meters_per_second = None
        
        self.__newton_seconds = None
        
        self.__pound_feet_per_second = None
        
        self.__pound_force_seconds = None
        
        self.__slug_feet_per_second = None
        
        self.__nanonewton_seconds = None
        
        self.__micronewton_seconds = None
        
        self.__millinewton_seconds = None
        
        self.__centinewton_seconds = None
        
        self.__decinewton_seconds = None
        
        self.__decanewton_seconds = None
        
        self.__kilonewton_seconds = None
        
        self.__meganewton_seconds = None
        

    def convert(self, unit: ImpulseUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ImpulseUnits = ImpulseUnits.NewtonSecond) -> ImpulseDto:
        """
        Get a new instance of Impulse DTO representing the current unit.

        :param hold_in_unit: The specific Impulse unit to store the Impulse value in the DTO representation.
        :type hold_in_unit: ImpulseUnits
        :return: A new instance of ImpulseDto.
        :rtype: ImpulseDto
        """
        return ImpulseDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ImpulseUnits = ImpulseUnits.NewtonSecond):
        """
        Get a Impulse DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Impulse unit to store the Impulse value in the DTO representation.
        :type hold_in_unit: ImpulseUnits
        :return: JSON object represents Impulse DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(impulse_dto: ImpulseDto):
        """
        Obtain a new instance of Impulse from a DTO unit object.

        :param impulse_dto: The Impulse DTO representation.
        :type impulse_dto: ImpulseDto
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(impulse_dto.value, impulse_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Impulse from a DTO unit json representation.

        :param data: The Impulse DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonSecond"}
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse.from_dto(ImpulseDto.from_json(data))

    def __convert_from_base(self, from_unit: ImpulseUnits) -> float:
        value = self._value
        
        if from_unit == ImpulseUnits.KilogramMeterPerSecond:
            return (value)
        
        if from_unit == ImpulseUnits.NewtonSecond:
            return (value)
        
        if from_unit == ImpulseUnits.PoundFootPerSecond:
            return (value * 7.230657989877)
        
        if from_unit == ImpulseUnits.PoundForceSecond:
            return (value * 0.2248089430997)
        
        if from_unit == ImpulseUnits.SlugFootPerSecond:
            return (value * 0.224735720691)
        
        if from_unit == ImpulseUnits.NanonewtonSecond:
            return ((value) / 1e-09)
        
        if from_unit == ImpulseUnits.MicronewtonSecond:
            return ((value) / 1e-06)
        
        if from_unit == ImpulseUnits.MillinewtonSecond:
            return ((value) / 0.001)
        
        if from_unit == ImpulseUnits.CentinewtonSecond:
            return ((value) / 0.01)
        
        if from_unit == ImpulseUnits.DecinewtonSecond:
            return ((value) / 0.1)
        
        if from_unit == ImpulseUnits.DecanewtonSecond:
            return ((value) / 10.0)
        
        if from_unit == ImpulseUnits.KilonewtonSecond:
            return ((value) / 1000.0)
        
        if from_unit == ImpulseUnits.MeganewtonSecond:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ImpulseUnits) -> float:
        
        if to_unit == ImpulseUnits.KilogramMeterPerSecond:
            return (value)
        
        if to_unit == ImpulseUnits.NewtonSecond:
            return (value)
        
        if to_unit == ImpulseUnits.PoundFootPerSecond:
            return (value / 7.230657989877)
        
        if to_unit == ImpulseUnits.PoundForceSecond:
            return (value / 0.2248089430997)
        
        if to_unit == ImpulseUnits.SlugFootPerSecond:
            return (value / 0.224735720691)
        
        if to_unit == ImpulseUnits.NanonewtonSecond:
            return ((value) * 1e-09)
        
        if to_unit == ImpulseUnits.MicronewtonSecond:
            return ((value) * 1e-06)
        
        if to_unit == ImpulseUnits.MillinewtonSecond:
            return ((value) * 0.001)
        
        if to_unit == ImpulseUnits.CentinewtonSecond:
            return ((value) * 0.01)
        
        if to_unit == ImpulseUnits.DecinewtonSecond:
            return ((value) * 0.1)
        
        if to_unit == ImpulseUnits.DecanewtonSecond:
            return ((value) * 10.0)
        
        if to_unit == ImpulseUnits.KilonewtonSecond:
            return ((value) * 1000.0)
        
        if to_unit == ImpulseUnits.MeganewtonSecond:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_kilogram_meters_per_second(kilogram_meters_per_second: float):
        """
        Create a new instance of Impulse from a value in kilogram_meters_per_second.

        

        :param meters: The Impulse value in kilogram_meters_per_second.
        :type kilogram_meters_per_second: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(kilogram_meters_per_second, ImpulseUnits.KilogramMeterPerSecond)

    
    @staticmethod
    def from_newton_seconds(newton_seconds: float):
        """
        Create a new instance of Impulse from a value in newton_seconds.

        

        :param meters: The Impulse value in newton_seconds.
        :type newton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(newton_seconds, ImpulseUnits.NewtonSecond)

    
    @staticmethod
    def from_pound_feet_per_second(pound_feet_per_second: float):
        """
        Create a new instance of Impulse from a value in pound_feet_per_second.

        

        :param meters: The Impulse value in pound_feet_per_second.
        :type pound_feet_per_second: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(pound_feet_per_second, ImpulseUnits.PoundFootPerSecond)

    
    @staticmethod
    def from_pound_force_seconds(pound_force_seconds: float):
        """
        Create a new instance of Impulse from a value in pound_force_seconds.

        

        :param meters: The Impulse value in pound_force_seconds.
        :type pound_force_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(pound_force_seconds, ImpulseUnits.PoundForceSecond)

    
    @staticmethod
    def from_slug_feet_per_second(slug_feet_per_second: float):
        """
        Create a new instance of Impulse from a value in slug_feet_per_second.

        

        :param meters: The Impulse value in slug_feet_per_second.
        :type slug_feet_per_second: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(slug_feet_per_second, ImpulseUnits.SlugFootPerSecond)

    
    @staticmethod
    def from_nanonewton_seconds(nanonewton_seconds: float):
        """
        Create a new instance of Impulse from a value in nanonewton_seconds.

        

        :param meters: The Impulse value in nanonewton_seconds.
        :type nanonewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(nanonewton_seconds, ImpulseUnits.NanonewtonSecond)

    
    @staticmethod
    def from_micronewton_seconds(micronewton_seconds: float):
        """
        Create a new instance of Impulse from a value in micronewton_seconds.

        

        :param meters: The Impulse value in micronewton_seconds.
        :type micronewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(micronewton_seconds, ImpulseUnits.MicronewtonSecond)

    
    @staticmethod
    def from_millinewton_seconds(millinewton_seconds: float):
        """
        Create a new instance of Impulse from a value in millinewton_seconds.

        

        :param meters: The Impulse value in millinewton_seconds.
        :type millinewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(millinewton_seconds, ImpulseUnits.MillinewtonSecond)

    
    @staticmethod
    def from_centinewton_seconds(centinewton_seconds: float):
        """
        Create a new instance of Impulse from a value in centinewton_seconds.

        

        :param meters: The Impulse value in centinewton_seconds.
        :type centinewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(centinewton_seconds, ImpulseUnits.CentinewtonSecond)

    
    @staticmethod
    def from_decinewton_seconds(decinewton_seconds: float):
        """
        Create a new instance of Impulse from a value in decinewton_seconds.

        

        :param meters: The Impulse value in decinewton_seconds.
        :type decinewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(decinewton_seconds, ImpulseUnits.DecinewtonSecond)

    
    @staticmethod
    def from_decanewton_seconds(decanewton_seconds: float):
        """
        Create a new instance of Impulse from a value in decanewton_seconds.

        

        :param meters: The Impulse value in decanewton_seconds.
        :type decanewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(decanewton_seconds, ImpulseUnits.DecanewtonSecond)

    
    @staticmethod
    def from_kilonewton_seconds(kilonewton_seconds: float):
        """
        Create a new instance of Impulse from a value in kilonewton_seconds.

        

        :param meters: The Impulse value in kilonewton_seconds.
        :type kilonewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(kilonewton_seconds, ImpulseUnits.KilonewtonSecond)

    
    @staticmethod
    def from_meganewton_seconds(meganewton_seconds: float):
        """
        Create a new instance of Impulse from a value in meganewton_seconds.

        

        :param meters: The Impulse value in meganewton_seconds.
        :type meganewton_seconds: float
        :return: A new instance of Impulse.
        :rtype: Impulse
        """
        return Impulse(meganewton_seconds, ImpulseUnits.MeganewtonSecond)

    
    @property
    def kilogram_meters_per_second(self) -> float:
        """
        
        """
        if self.__kilogram_meters_per_second != None:
            return self.__kilogram_meters_per_second
        self.__kilogram_meters_per_second = self.__convert_from_base(ImpulseUnits.KilogramMeterPerSecond)
        return self.__kilogram_meters_per_second

    
    @property
    def newton_seconds(self) -> float:
        """
        
        """
        if self.__newton_seconds != None:
            return self.__newton_seconds
        self.__newton_seconds = self.__convert_from_base(ImpulseUnits.NewtonSecond)
        return self.__newton_seconds

    
    @property
    def pound_feet_per_second(self) -> float:
        """
        
        """
        if self.__pound_feet_per_second != None:
            return self.__pound_feet_per_second
        self.__pound_feet_per_second = self.__convert_from_base(ImpulseUnits.PoundFootPerSecond)
        return self.__pound_feet_per_second

    
    @property
    def pound_force_seconds(self) -> float:
        """
        
        """
        if self.__pound_force_seconds != None:
            return self.__pound_force_seconds
        self.__pound_force_seconds = self.__convert_from_base(ImpulseUnits.PoundForceSecond)
        return self.__pound_force_seconds

    
    @property
    def slug_feet_per_second(self) -> float:
        """
        
        """
        if self.__slug_feet_per_second != None:
            return self.__slug_feet_per_second
        self.__slug_feet_per_second = self.__convert_from_base(ImpulseUnits.SlugFootPerSecond)
        return self.__slug_feet_per_second

    
    @property
    def nanonewton_seconds(self) -> float:
        """
        
        """
        if self.__nanonewton_seconds != None:
            return self.__nanonewton_seconds
        self.__nanonewton_seconds = self.__convert_from_base(ImpulseUnits.NanonewtonSecond)
        return self.__nanonewton_seconds

    
    @property
    def micronewton_seconds(self) -> float:
        """
        
        """
        if self.__micronewton_seconds != None:
            return self.__micronewton_seconds
        self.__micronewton_seconds = self.__convert_from_base(ImpulseUnits.MicronewtonSecond)
        return self.__micronewton_seconds

    
    @property
    def millinewton_seconds(self) -> float:
        """
        
        """
        if self.__millinewton_seconds != None:
            return self.__millinewton_seconds
        self.__millinewton_seconds = self.__convert_from_base(ImpulseUnits.MillinewtonSecond)
        return self.__millinewton_seconds

    
    @property
    def centinewton_seconds(self) -> float:
        """
        
        """
        if self.__centinewton_seconds != None:
            return self.__centinewton_seconds
        self.__centinewton_seconds = self.__convert_from_base(ImpulseUnits.CentinewtonSecond)
        return self.__centinewton_seconds

    
    @property
    def decinewton_seconds(self) -> float:
        """
        
        """
        if self.__decinewton_seconds != None:
            return self.__decinewton_seconds
        self.__decinewton_seconds = self.__convert_from_base(ImpulseUnits.DecinewtonSecond)
        return self.__decinewton_seconds

    
    @property
    def decanewton_seconds(self) -> float:
        """
        
        """
        if self.__decanewton_seconds != None:
            return self.__decanewton_seconds
        self.__decanewton_seconds = self.__convert_from_base(ImpulseUnits.DecanewtonSecond)
        return self.__decanewton_seconds

    
    @property
    def kilonewton_seconds(self) -> float:
        """
        
        """
        if self.__kilonewton_seconds != None:
            return self.__kilonewton_seconds
        self.__kilonewton_seconds = self.__convert_from_base(ImpulseUnits.KilonewtonSecond)
        return self.__kilonewton_seconds

    
    @property
    def meganewton_seconds(self) -> float:
        """
        
        """
        if self.__meganewton_seconds != None:
            return self.__meganewton_seconds
        self.__meganewton_seconds = self.__convert_from_base(ImpulseUnits.MeganewtonSecond)
        return self.__meganewton_seconds

    
    def to_string(self, unit: ImpulseUnits = ImpulseUnits.NewtonSecond, fractional_digits: int = None) -> str:
        """
        Format the Impulse to a string.
        
        Note: the default format for Impulse is NewtonSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Impulse. Default is 'NewtonSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ImpulseUnits.KilogramMeterPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilogram_meters_per_second, fractional_digits)} kg·m/s"""
        
        if unit == ImpulseUnits.NewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.newton_seconds, fractional_digits)} N·s"""
        
        if unit == ImpulseUnits.PoundFootPerSecond:
            return f"""{super()._truncate_fraction_digits(self.pound_feet_per_second, fractional_digits)} lb·ft/s"""
        
        if unit == ImpulseUnits.PoundForceSecond:
            return f"""{super()._truncate_fraction_digits(self.pound_force_seconds, fractional_digits)} lbf·s"""
        
        if unit == ImpulseUnits.SlugFootPerSecond:
            return f"""{super()._truncate_fraction_digits(self.slug_feet_per_second, fractional_digits)} slug·ft/s"""
        
        if unit == ImpulseUnits.NanonewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.nanonewton_seconds, fractional_digits)} nN·s"""
        
        if unit == ImpulseUnits.MicronewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.micronewton_seconds, fractional_digits)} μN·s"""
        
        if unit == ImpulseUnits.MillinewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.millinewton_seconds, fractional_digits)} mN·s"""
        
        if unit == ImpulseUnits.CentinewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.centinewton_seconds, fractional_digits)} cN·s"""
        
        if unit == ImpulseUnits.DecinewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.decinewton_seconds, fractional_digits)} dN·s"""
        
        if unit == ImpulseUnits.DecanewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.decanewton_seconds, fractional_digits)} daN·s"""
        
        if unit == ImpulseUnits.KilonewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_seconds, fractional_digits)} kN·s"""
        
        if unit == ImpulseUnits.MeganewtonSecond:
            return f"""{super()._truncate_fraction_digits(self.meganewton_seconds, fractional_digits)} MN·s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ImpulseUnits = ImpulseUnits.NewtonSecond) -> str:
        """
        Get Impulse unit abbreviation.
        Note! the default abbreviation for Impulse is NewtonSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ImpulseUnits.KilogramMeterPerSecond:
            return """kg·m/s"""
        
        if unit_abbreviation == ImpulseUnits.NewtonSecond:
            return """N·s"""
        
        if unit_abbreviation == ImpulseUnits.PoundFootPerSecond:
            return """lb·ft/s"""
        
        if unit_abbreviation == ImpulseUnits.PoundForceSecond:
            return """lbf·s"""
        
        if unit_abbreviation == ImpulseUnits.SlugFootPerSecond:
            return """slug·ft/s"""
        
        if unit_abbreviation == ImpulseUnits.NanonewtonSecond:
            return """nN·s"""
        
        if unit_abbreviation == ImpulseUnits.MicronewtonSecond:
            return """μN·s"""
        
        if unit_abbreviation == ImpulseUnits.MillinewtonSecond:
            return """mN·s"""
        
        if unit_abbreviation == ImpulseUnits.CentinewtonSecond:
            return """cN·s"""
        
        if unit_abbreviation == ImpulseUnits.DecinewtonSecond:
            return """dN·s"""
        
        if unit_abbreviation == ImpulseUnits.DecanewtonSecond:
            return """daN·s"""
        
        if unit_abbreviation == ImpulseUnits.KilonewtonSecond:
            return """kN·s"""
        
        if unit_abbreviation == ImpulseUnits.MeganewtonSecond:
            return """MN·s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class LuminousFluxUnits(Enum):
        """
            LuminousFluxUnits enumeration
        """
        
        Lumen = 'Lumen'
        """
            
        """
        

class LuminousFluxDto:
    """
    A DTO representation of a LuminousFlux

    Attributes:
        value (float): The value of the LuminousFlux.
        unit (LuminousFluxUnits): The specific unit that the LuminousFlux value is representing.
    """

    def __init__(self, value: float, unit: LuminousFluxUnits):
        """
        Create a new DTO representation of a LuminousFlux

        Parameters:
            value (float): The value of the LuminousFlux.
            unit (LuminousFluxUnits): The specific unit that the LuminousFlux value is representing.
        """
        self.value: float = value
        """
        The value of the LuminousFlux
        """
        self.unit: LuminousFluxUnits = unit
        """
        The specific unit that the LuminousFlux value is representing
        """

    def to_json(self):
        """
        Get a LuminousFlux DTO JSON object representing the current unit.

        :return: JSON object represents LuminousFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Lumen"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of LuminousFlux DTO from a json representation.

        :param data: The LuminousFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Lumen"}
        :return: A new instance of LuminousFluxDto.
        :rtype: LuminousFluxDto
        """
        return LuminousFluxDto(value=data["value"], unit=LuminousFluxUnits(data["unit"]))


class LuminousFlux(AbstractMeasure):
    """
    In photometry, luminous flux or luminous power is the measure of the perceived power of light.

    Args:
        value (float): The value.
        from_unit (LuminousFluxUnits): The LuminousFlux unit to create from, The default unit is Lumen
    """
    def __init__(self, value: float, from_unit: LuminousFluxUnits = LuminousFluxUnits.Lumen):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__lumens = None
        

    def convert(self, unit: LuminousFluxUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: LuminousFluxUnits = LuminousFluxUnits.Lumen) -> LuminousFluxDto:
        """
        Get a new instance of LuminousFlux DTO representing the current unit.

        :param hold_in_unit: The specific LuminousFlux unit to store the LuminousFlux value in the DTO representation.
        :type hold_in_unit: LuminousFluxUnits
        :return: A new instance of LuminousFluxDto.
        :rtype: LuminousFluxDto
        """
        return LuminousFluxDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: LuminousFluxUnits = LuminousFluxUnits.Lumen):
        """
        Get a LuminousFlux DTO JSON object representing the current unit.

        :param hold_in_unit: The specific LuminousFlux unit to store the LuminousFlux value in the DTO representation.
        :type hold_in_unit: LuminousFluxUnits
        :return: JSON object represents LuminousFlux DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Lumen"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(luminous_flux_dto: LuminousFluxDto):
        """
        Obtain a new instance of LuminousFlux from a DTO unit object.

        :param luminous_flux_dto: The LuminousFlux DTO representation.
        :type luminous_flux_dto: LuminousFluxDto
        :return: A new instance of LuminousFlux.
        :rtype: LuminousFlux
        """
        return LuminousFlux(luminous_flux_dto.value, luminous_flux_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of LuminousFlux from a DTO unit json representation.

        :param data: The LuminousFlux DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Lumen"}
        :return: A new instance of LuminousFlux.
        :rtype: LuminousFlux
        """
        return LuminousFlux.from_dto(LuminousFluxDto.from_json(data))

    def __convert_from_base(self, from_unit: LuminousFluxUnits) -> float:
        value = self._value
        
        if from_unit == LuminousFluxUnits.Lumen:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LuminousFluxUnits) -> float:
        
        if to_unit == LuminousFluxUnits.Lumen:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_lumens(lumens: float):
        """
        Create a new instance of LuminousFlux from a value in lumens.

        

        :param meters: The LuminousFlux value in lumens.
        :type lumens: float
        :return: A new instance of LuminousFlux.
        :rtype: LuminousFlux
        """
        return LuminousFlux(lumens, LuminousFluxUnits.Lumen)

    
    @property
    def lumens(self) -> float:
        """
        
        """
        if self.__lumens != None:
            return self.__lumens
        self.__lumens = self.__convert_from_base(LuminousFluxUnits.Lumen)
        return self.__lumens

    
    def to_string(self, unit: LuminousFluxUnits = LuminousFluxUnits.Lumen, fractional_digits: int = None) -> str:
        """
        Format the LuminousFlux to a string.
        
        Note: the default format for LuminousFlux is Lumen.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the LuminousFlux. Default is 'Lumen'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == LuminousFluxUnits.Lumen:
            return f"""{super()._truncate_fraction_digits(self.lumens, fractional_digits)} lm"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: LuminousFluxUnits = LuminousFluxUnits.Lumen) -> str:
        """
        Get LuminousFlux unit abbreviation.
        Note! the default abbreviation for LuminousFlux is Lumen.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LuminousFluxUnits.Lumen:
            return """lm"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MolarFlowUnits(Enum):
        """
            MolarFlowUnits enumeration
        """
        
        MolePerSecond = 'MolePerSecond'
        """
            
        """
        
        MolePerMinute = 'MolePerMinute'
        """
            
        """
        
        MolePerHour = 'MolePerHour'
        """
            
        """
        
        PoundMolePerSecond = 'PoundMolePerSecond'
        """
            
        """
        
        PoundMolePerMinute = 'PoundMolePerMinute'
        """
            
        """
        
        PoundMolePerHour = 'PoundMolePerHour'
        """
            
        """
        
        KilomolePerSecond = 'KilomolePerSecond'
        """
            
        """
        
        KilomolePerMinute = 'KilomolePerMinute'
        """
            
        """
        
        KilomolePerHour = 'KilomolePerHour'
        """
            
        """
        

class MolarFlowDto:
    """
    A DTO representation of a MolarFlow

    Attributes:
        value (float): The value of the MolarFlow.
        unit (MolarFlowUnits): The specific unit that the MolarFlow value is representing.
    """

    def __init__(self, value: float, unit: MolarFlowUnits):
        """
        Create a new DTO representation of a MolarFlow

        Parameters:
            value (float): The value of the MolarFlow.
            unit (MolarFlowUnits): The specific unit that the MolarFlow value is representing.
        """
        self.value: float = value
        """
        The value of the MolarFlow
        """
        self.unit: MolarFlowUnits = unit
        """
        The specific unit that the MolarFlow value is representing
        """

    def to_json(self):
        """
        Get a MolarFlow DTO JSON object representing the current unit.

        :return: JSON object represents MolarFlow DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MolePerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MolarFlow DTO from a json representation.

        :param data: The MolarFlow DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MolePerSecond"}
        :return: A new instance of MolarFlowDto.
        :rtype: MolarFlowDto
        """
        return MolarFlowDto(value=data["value"], unit=MolarFlowUnits(data["unit"]))


class MolarFlow(AbstractMeasure):
    """
    Molar flow is the ratio of the amount of substance change to the time during which the change occurred (value of amount of substance changes per unit time).

    Args:
        value (float): The value.
        from_unit (MolarFlowUnits): The MolarFlow unit to create from, The default unit is MolePerSecond
    """
    def __init__(self, value: float, from_unit: MolarFlowUnits = MolarFlowUnits.MolePerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__moles_per_second = None
        
        self.__moles_per_minute = None
        
        self.__moles_per_hour = None
        
        self.__pound_moles_per_second = None
        
        self.__pound_moles_per_minute = None
        
        self.__pound_moles_per_hour = None
        
        self.__kilomoles_per_second = None
        
        self.__kilomoles_per_minute = None
        
        self.__kilomoles_per_hour = None
        

    def convert(self, unit: MolarFlowUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MolarFlowUnits = MolarFlowUnits.MolePerSecond) -> MolarFlowDto:
        """
        Get a new instance of MolarFlow DTO representing the current unit.

        :param hold_in_unit: The specific MolarFlow unit to store the MolarFlow value in the DTO representation.
        :type hold_in_unit: MolarFlowUnits
        :return: A new instance of MolarFlowDto.
        :rtype: MolarFlowDto
        """
        return MolarFlowDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MolarFlowUnits = MolarFlowUnits.MolePerSecond):
        """
        Get a MolarFlow DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MolarFlow unit to store the MolarFlow value in the DTO representation.
        :type hold_in_unit: MolarFlowUnits
        :return: JSON object represents MolarFlow DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MolePerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(molar_flow_dto: MolarFlowDto):
        """
        Obtain a new instance of MolarFlow from a DTO unit object.

        :param molar_flow_dto: The MolarFlow DTO representation.
        :type molar_flow_dto: MolarFlowDto
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(molar_flow_dto.value, molar_flow_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MolarFlow from a DTO unit json representation.

        :param data: The MolarFlow DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MolePerSecond"}
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow.from_dto(MolarFlowDto.from_json(data))

    def __convert_from_base(self, from_unit: MolarFlowUnits) -> float:
        value = self._value
        
        if from_unit == MolarFlowUnits.MolePerSecond:
            return (value)
        
        if from_unit == MolarFlowUnits.MolePerMinute:
            return (value * 60)
        
        if from_unit == MolarFlowUnits.MolePerHour:
            return (value * 3600)
        
        if from_unit == MolarFlowUnits.PoundMolePerSecond:
            return (value / 453.59237)
        
        if from_unit == MolarFlowUnits.PoundMolePerMinute:
            return ((value / 453.59237) * 60)
        
        if from_unit == MolarFlowUnits.PoundMolePerHour:
            return ((value / 453.59237) * 3600)
        
        if from_unit == MolarFlowUnits.KilomolePerSecond:
            return ((value) / 1000.0)
        
        if from_unit == MolarFlowUnits.KilomolePerMinute:
            return ((value * 60) / 1000.0)
        
        if from_unit == MolarFlowUnits.KilomolePerHour:
            return ((value * 3600) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolarFlowUnits) -> float:
        
        if to_unit == MolarFlowUnits.MolePerSecond:
            return (value)
        
        if to_unit == MolarFlowUnits.MolePerMinute:
            return (value / 60)
        
        if to_unit == MolarFlowUnits.MolePerHour:
            return (value / 3600)
        
        if to_unit == MolarFlowUnits.PoundMolePerSecond:
            return (value * 453.59237)
        
        if to_unit == MolarFlowUnits.PoundMolePerMinute:
            return ((value * 453.59237) / 60)
        
        if to_unit == MolarFlowUnits.PoundMolePerHour:
            return ((value * 453.59237) / 3600)
        
        if to_unit == MolarFlowUnits.KilomolePerSecond:
            return ((value) * 1000.0)
        
        if to_unit == MolarFlowUnits.KilomolePerMinute:
            return ((value / 60) * 1000.0)
        
        if to_unit == MolarFlowUnits.KilomolePerHour:
            return ((value / 3600) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_moles_per_second(moles_per_second: float):
        """
        Create a new instance of MolarFlow from a value in moles_per_second.

        

        :param meters: The MolarFlow value in moles_per_second.
        :type moles_per_second: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(moles_per_second, MolarFlowUnits.MolePerSecond)

    
    @staticmethod
    def from_moles_per_minute(moles_per_minute: float):
        """
        Create a new instance of MolarFlow from a value in moles_per_minute.

        

        :param meters: The MolarFlow value in moles_per_minute.
        :type moles_per_minute: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(moles_per_minute, MolarFlowUnits.MolePerMinute)

    
    @staticmethod
    def from_moles_per_hour(moles_per_hour: float):
        """
        Create a new instance of MolarFlow from a value in moles_per_hour.

        

        :param meters: The MolarFlow value in moles_per_hour.
        :type moles_per_hour: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(moles_per_hour, MolarFlowUnits.MolePerHour)

    
    @staticmethod
    def from_pound_moles_per_second(pound_moles_per_second: float):
        """
        Create a new instance of MolarFlow from a value in pound_moles_per_second.

        

        :param meters: The MolarFlow value in pound_moles_per_second.
        :type pound_moles_per_second: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(pound_moles_per_second, MolarFlowUnits.PoundMolePerSecond)

    
    @staticmethod
    def from_pound_moles_per_minute(pound_moles_per_minute: float):
        """
        Create a new instance of MolarFlow from a value in pound_moles_per_minute.

        

        :param meters: The MolarFlow value in pound_moles_per_minute.
        :type pound_moles_per_minute: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(pound_moles_per_minute, MolarFlowUnits.PoundMolePerMinute)

    
    @staticmethod
    def from_pound_moles_per_hour(pound_moles_per_hour: float):
        """
        Create a new instance of MolarFlow from a value in pound_moles_per_hour.

        

        :param meters: The MolarFlow value in pound_moles_per_hour.
        :type pound_moles_per_hour: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(pound_moles_per_hour, MolarFlowUnits.PoundMolePerHour)

    
    @staticmethod
    def from_kilomoles_per_second(kilomoles_per_second: float):
        """
        Create a new instance of MolarFlow from a value in kilomoles_per_second.

        

        :param meters: The MolarFlow value in kilomoles_per_second.
        :type kilomoles_per_second: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(kilomoles_per_second, MolarFlowUnits.KilomolePerSecond)

    
    @staticmethod
    def from_kilomoles_per_minute(kilomoles_per_minute: float):
        """
        Create a new instance of MolarFlow from a value in kilomoles_per_minute.

        

        :param meters: The MolarFlow value in kilomoles_per_minute.
        :type kilomoles_per_minute: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(kilomoles_per_minute, MolarFlowUnits.KilomolePerMinute)

    
    @staticmethod
    def from_kilomoles_per_hour(kilomoles_per_hour: float):
        """
        Create a new instance of MolarFlow from a value in kilomoles_per_hour.

        

        :param meters: The MolarFlow value in kilomoles_per_hour.
        :type kilomoles_per_hour: float
        :return: A new instance of MolarFlow.
        :rtype: MolarFlow
        """
        return MolarFlow(kilomoles_per_hour, MolarFlowUnits.KilomolePerHour)

    
    @property
    def moles_per_second(self) -> float:
        """
        
        """
        if self.__moles_per_second != None:
            return self.__moles_per_second
        self.__moles_per_second = self.__convert_from_base(MolarFlowUnits.MolePerSecond)
        return self.__moles_per_second

    
    @property
    def moles_per_minute(self) -> float:
        """
        
        """
        if self.__moles_per_minute != None:
            return self.__moles_per_minute
        self.__moles_per_minute = self.__convert_from_base(MolarFlowUnits.MolePerMinute)
        return self.__moles_per_minute

    
    @property
    def moles_per_hour(self) -> float:
        """
        
        """
        if self.__moles_per_hour != None:
            return self.__moles_per_hour
        self.__moles_per_hour = self.__convert_from_base(MolarFlowUnits.MolePerHour)
        return self.__moles_per_hour

    
    @property
    def pound_moles_per_second(self) -> float:
        """
        
        """
        if self.__pound_moles_per_second != None:
            return self.__pound_moles_per_second
        self.__pound_moles_per_second = self.__convert_from_base(MolarFlowUnits.PoundMolePerSecond)
        return self.__pound_moles_per_second

    
    @property
    def pound_moles_per_minute(self) -> float:
        """
        
        """
        if self.__pound_moles_per_minute != None:
            return self.__pound_moles_per_minute
        self.__pound_moles_per_minute = self.__convert_from_base(MolarFlowUnits.PoundMolePerMinute)
        return self.__pound_moles_per_minute

    
    @property
    def pound_moles_per_hour(self) -> float:
        """
        
        """
        if self.__pound_moles_per_hour != None:
            return self.__pound_moles_per_hour
        self.__pound_moles_per_hour = self.__convert_from_base(MolarFlowUnits.PoundMolePerHour)
        return self.__pound_moles_per_hour

    
    @property
    def kilomoles_per_second(self) -> float:
        """
        
        """
        if self.__kilomoles_per_second != None:
            return self.__kilomoles_per_second
        self.__kilomoles_per_second = self.__convert_from_base(MolarFlowUnits.KilomolePerSecond)
        return self.__kilomoles_per_second

    
    @property
    def kilomoles_per_minute(self) -> float:
        """
        
        """
        if self.__kilomoles_per_minute != None:
            return self.__kilomoles_per_minute
        self.__kilomoles_per_minute = self.__convert_from_base(MolarFlowUnits.KilomolePerMinute)
        return self.__kilomoles_per_minute

    
    @property
    def kilomoles_per_hour(self) -> float:
        """
        
        """
        if self.__kilomoles_per_hour != None:
            return self.__kilomoles_per_hour
        self.__kilomoles_per_hour = self.__convert_from_base(MolarFlowUnits.KilomolePerHour)
        return self.__kilomoles_per_hour

    
    def to_string(self, unit: MolarFlowUnits = MolarFlowUnits.MolePerSecond, fractional_digits: int = None) -> str:
        """
        Format the MolarFlow to a string.
        
        Note: the default format for MolarFlow is MolePerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MolarFlow. Default is 'MolePerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MolarFlowUnits.MolePerSecond:
            return f"""{super()._truncate_fraction_digits(self.moles_per_second, fractional_digits)} mol/s"""
        
        if unit == MolarFlowUnits.MolePerMinute:
            return f"""{super()._truncate_fraction_digits(self.moles_per_minute, fractional_digits)} mol/min"""
        
        if unit == MolarFlowUnits.MolePerHour:
            return f"""{super()._truncate_fraction_digits(self.moles_per_hour, fractional_digits)} kmol/h"""
        
        if unit == MolarFlowUnits.PoundMolePerSecond:
            return f"""{super()._truncate_fraction_digits(self.pound_moles_per_second, fractional_digits)} lbmol/s"""
        
        if unit == MolarFlowUnits.PoundMolePerMinute:
            return f"""{super()._truncate_fraction_digits(self.pound_moles_per_minute, fractional_digits)} lbmol/min"""
        
        if unit == MolarFlowUnits.PoundMolePerHour:
            return f"""{super()._truncate_fraction_digits(self.pound_moles_per_hour, fractional_digits)} lbmol/h"""
        
        if unit == MolarFlowUnits.KilomolePerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilomoles_per_second, fractional_digits)} kmol/s"""
        
        if unit == MolarFlowUnits.KilomolePerMinute:
            return f"""{super()._truncate_fraction_digits(self.kilomoles_per_minute, fractional_digits)} kmol/min"""
        
        if unit == MolarFlowUnits.KilomolePerHour:
            return f"""{super()._truncate_fraction_digits(self.kilomoles_per_hour, fractional_digits)} kkmol/h"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolarFlowUnits = MolarFlowUnits.MolePerSecond) -> str:
        """
        Get MolarFlow unit abbreviation.
        Note! the default abbreviation for MolarFlow is MolePerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolarFlowUnits.MolePerSecond:
            return """mol/s"""
        
        if unit_abbreviation == MolarFlowUnits.MolePerMinute:
            return """mol/min"""
        
        if unit_abbreviation == MolarFlowUnits.MolePerHour:
            return """kmol/h"""
        
        if unit_abbreviation == MolarFlowUnits.PoundMolePerSecond:
            return """lbmol/s"""
        
        if unit_abbreviation == MolarFlowUnits.PoundMolePerMinute:
            return """lbmol/min"""
        
        if unit_abbreviation == MolarFlowUnits.PoundMolePerHour:
            return """lbmol/h"""
        
        if unit_abbreviation == MolarFlowUnits.KilomolePerSecond:
            return """kmol/s"""
        
        if unit_abbreviation == MolarFlowUnits.KilomolePerMinute:
            return """kmol/min"""
        
        if unit_abbreviation == MolarFlowUnits.KilomolePerHour:
            return """kkmol/h"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MagnetizationUnits(Enum):
        """
            MagnetizationUnits enumeration
        """
        
        AmperePerMeter = 'AmperePerMeter'
        """
            
        """
        

class MagnetizationDto:
    """
    A DTO representation of a Magnetization

    Attributes:
        value (float): The value of the Magnetization.
        unit (MagnetizationUnits): The specific unit that the Magnetization value is representing.
    """

    def __init__(self, value: float, unit: MagnetizationUnits):
        """
        Create a new DTO representation of a Magnetization

        Parameters:
            value (float): The value of the Magnetization.
            unit (MagnetizationUnits): The specific unit that the Magnetization value is representing.
        """
        self.value: float = value
        """
        The value of the Magnetization
        """
        self.unit: MagnetizationUnits = unit
        """
        The specific unit that the Magnetization value is representing
        """

    def to_json(self):
        """
        Get a Magnetization DTO JSON object representing the current unit.

        :return: JSON object represents Magnetization DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "AmperePerMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Magnetization DTO from a json representation.

        :param data: The Magnetization DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "AmperePerMeter"}
        :return: A new instance of MagnetizationDto.
        :rtype: MagnetizationDto
        """
        return MagnetizationDto(value=data["value"], unit=MagnetizationUnits(data["unit"]))


class Magnetization(AbstractMeasure):
    """
    In classical electromagnetism, magnetization is the vector field that expresses the density of permanent or induced magnetic dipole moments in a magnetic material.

    Args:
        value (float): The value.
        from_unit (MagnetizationUnits): The Magnetization unit to create from, The default unit is AmperePerMeter
    """
    def __init__(self, value: float, from_unit: MagnetizationUnits = MagnetizationUnits.AmperePerMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__amperes_per_meter = None
        

    def convert(self, unit: MagnetizationUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MagnetizationUnits = MagnetizationUnits.AmperePerMeter) -> MagnetizationDto:
        """
        Get a new instance of Magnetization DTO representing the current unit.

        :param hold_in_unit: The specific Magnetization unit to store the Magnetization value in the DTO representation.
        :type hold_in_unit: MagnetizationUnits
        :return: A new instance of MagnetizationDto.
        :rtype: MagnetizationDto
        """
        return MagnetizationDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MagnetizationUnits = MagnetizationUnits.AmperePerMeter):
        """
        Get a Magnetization DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Magnetization unit to store the Magnetization value in the DTO representation.
        :type hold_in_unit: MagnetizationUnits
        :return: JSON object represents Magnetization DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "AmperePerMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(magnetization_dto: MagnetizationDto):
        """
        Obtain a new instance of Magnetization from a DTO unit object.

        :param magnetization_dto: The Magnetization DTO representation.
        :type magnetization_dto: MagnetizationDto
        :return: A new instance of Magnetization.
        :rtype: Magnetization
        """
        return Magnetization(magnetization_dto.value, magnetization_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Magnetization from a DTO unit json representation.

        :param data: The Magnetization DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "AmperePerMeter"}
        :return: A new instance of Magnetization.
        :rtype: Magnetization
        """
        return Magnetization.from_dto(MagnetizationDto.from_json(data))

    def __convert_from_base(self, from_unit: MagnetizationUnits) -> float:
        value = self._value
        
        if from_unit == MagnetizationUnits.AmperePerMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MagnetizationUnits) -> float:
        
        if to_unit == MagnetizationUnits.AmperePerMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_amperes_per_meter(amperes_per_meter: float):
        """
        Create a new instance of Magnetization from a value in amperes_per_meter.

        

        :param meters: The Magnetization value in amperes_per_meter.
        :type amperes_per_meter: float
        :return: A new instance of Magnetization.
        :rtype: Magnetization
        """
        return Magnetization(amperes_per_meter, MagnetizationUnits.AmperePerMeter)

    
    @property
    def amperes_per_meter(self) -> float:
        """
        
        """
        if self.__amperes_per_meter != None:
            return self.__amperes_per_meter
        self.__amperes_per_meter = self.__convert_from_base(MagnetizationUnits.AmperePerMeter)
        return self.__amperes_per_meter

    
    def to_string(self, unit: MagnetizationUnits = MagnetizationUnits.AmperePerMeter, fractional_digits: int = None) -> str:
        """
        Format the Magnetization to a string.
        
        Note: the default format for Magnetization is AmperePerMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Magnetization. Default is 'AmperePerMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MagnetizationUnits.AmperePerMeter:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_meter, fractional_digits)} A/m"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MagnetizationUnits = MagnetizationUnits.AmperePerMeter) -> str:
        """
        Get Magnetization unit abbreviation.
        Note! the default abbreviation for Magnetization is AmperePerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MagnetizationUnits.AmperePerMeter:
            return """A/m"""
        